/**
 * 
 */
package com.amx.duet.tools.comm;

/**
 * @author mmanners
 *
 */
public interface ISurroundModes
{

	/**
	 * Returns the number of valid surround mode settings.
	 * 
	 * @return The number of surround mode settings.
	 */
	public abstract int getSurroundModeCount();
	
	/**
	 * Returns the properties for all surround mode settings.
	 * 
	 * @return All the properties for the surround modes.
	 */	
	public abstract String[] getSurroundModeProperties();

	/**
	 * Returns the properties for a single surround mode setting.
	 * 
	 * @param surroundModeNumber integer value representing the selectable surround mode
	 * @return String
 	 * @return Formatted property string.
	 */
	public String getSurroundModeProperty(int surroundModeNumber);

	/**
	 * Returns the currently selected surround mode.
	 * 
	 * @return int
	 */
	public abstract int getSurroundModeSelect();

	/**
	 * Processes the event when the PreAmp Surround Mode Count is requested.<br>
	 * @param surroundModeNumber integer value representing the selectable surround mode
	 */
	public abstract void processSurroundModeCountEvent(int surroundModeNumber);

	/**
	 * Processes the event when the PreAmp Surround Mode properties are requested.<br>
	 * @param propertyArray
	 */
	public abstract void processSurroundModeProperties(String[] propertyArray);

	/**
	 * Processes the event when the PreAmp Surround Mode Property is changed.<br>
	 * @param propertyValue 
	 */
	public abstract void processSurroundModePropertyEvent(String propertyValue);

	/**
	 * Processes the event when the PreAmp Surround Mode selection is made.<br>
	 * @param surroundModeNumber integer value representing the selectable surround mode
	 */
	public abstract void processSurroundModeSelectEvent(int surroundModeNumber);

	/**
	 * Sets the current surround mode setting, where index is a virtual index number between 1 and
	 * the value returned by getSurroundModeCount().
	 * 
	 * @param surroundModeNumber integer value representing the selectable surround mode
	 */
	public abstract void setSurroundModeSelect(int surroundModeNumber);

	/**
	 * Set the SurroundMode for the device.<br>
	 * The new value is compared against the current value.  If the value has changed, the new
	 * value is saved and a process event for the SurroundMode is called.<br>
	 * @param surroundModeNumber the number of the surround mode selected 
	 */
	public abstract void updateSurroundModeSelect( int surroundModeNumber );


}
