/*
 * Created on May 15, 2005
 * 
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.io;


/**
 * Interface used to differentiate marshalled data types from marshalled classes.
 * If you want to marshall a class simply implement this interface and the marshalling
 * functions
 * 
 * <P>Here's an example of a marshalled class:
 * 
 * <PRE>
 * public class RoomDetails implements <STRONG>IStruct</STRONG>
 * {
 * 	public boolean bValid = false;
 * 	public String sPrestigeLevel = "";
 * 	public int nNumberSeats = 0;
 * 
 * 	public int <STRONG>binaryEncode</STRONG>(ObjectRef buffer, ObjectRef rPos)
 * 	{
 * 		int ret = 0;
 * 		ByteArrayOutputStream stream = new ByteArrayOutputStream();
 * 		ObjectRef encoder = new ObjectRef(0);
 * 
 * 		// Write start
 * 		stream.write(<STRONG>TYPE_STRUCT_START</STRONG>);
 * 		buffer.valueOf(buffer.toString() + stream.toString());
 * 		rPos.valueOf(rPos.toInt() + 1);
 * 
 * 		encoder.valueOf(bValid);
 * 		if ((ret = NetLinx.<STRONG>VariableToString</STRONG>(encoder, buffer, rPos)) < 0) return ret;
 * 
 * 		encoder.valueOf(sPrestigeLevel);
 * 		if ((ret = NetLinx.<STRONG>VariableToString</STRONG>(encoder, buffer, rPos)) < 0) return ret;
 * 
 * 		encoder.valueOf((short)nNumberSeats);
 * 		if ((ret = NetLinx.<STRONG>VariableToString</STRONG>(encoder, buffer, rPos)) < 0) return ret;
 * 
 * 		// Write end
 * 		stream.reset();
 * 		stream.write(<STRONG>TYPE_STRUCT_END</STRONG>);
 * 		buffer.valueOf(buffer.toString() + stream.toString());
 * 		rPos.valueOf(rPos.toInt() + 1);
 * 
 * 		return ret;
 * 	}
 * 	public int <STRONG>binaryDecode</STRONG>(ObjectRef buffer, ObjectRef rPos)
 * 	{
 * 		int ret = 0;
 * 		ObjectRef decoder = new ObjectRef(0);
 * 		rPos.valueOf(rPos.toInt() + 1);
 * 
 * 		decoder.valueOf(bValid);
 * 		if ((ret = NetLinx.<STRONG>StringToVariable</STRONG>(decoder, buffer, rPos)) < 0) return ret;
 * 		bValid = decoder.toBoolean();
 * 
 * 		decoder.valueOf(sPrestigeLevel);
 * 		if ((ret = NetLinx.<STRONG>StringToVariable</STRONG>(decoder, buffer, rPos)) < 0) return ret;
 * 		sPrestigeLevel = decoder.toString();
 * 
 * 		decoder.valueOf((short)nNumberSeats);
 * 		if ((ret = NetLinx.<STRONG>StringToVariable</STRONG>(decoder, buffer, rPos)) < 0) return ret;
 * 		nNumberSeats = decoder.toInt();
 * 
 * 		// don't forget the end of struct byte
 * 		rPos.valueOf(rPos.toInt() + 1);
 * 
 * 		return ret;
 * 	}
 * }
 * </PRE>
 *  
 * @version 1.0.0
 * @since AMXTools 1.0.0
 * 
 */
public interface IStruct extends IBinaryMarshall
{
	/**
	 * This method gives all IStruct objects the opporitunity to clean up
	 * their own member data
	 *
	 */
	public abstract void dispose();
}
