/*
 * BinaryMarshall.java
 * 
 * Created on Jul 7, 2006
 * Created by VGonzales
 *
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.io;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.UnsupportedEncodingException;

import com.amx.duet.tools.lang.Bits;
import com.amx.duet.tools.lang.ObjectRef;
import com.amx.duet.tools.lang.ObjectType;

/**
 * Includes handling for NetLinx binary marshalling.
 * 
 * @version 1.0.0
 */
public class Marshall extends ObjectRef implements IBinaryMarshall
{
    /**
     * Creates an object representation of the <code>boolean</code> argument. 
     *
     * @param   b   a <code>boolean</code>.
     */
	public Marshall(boolean b)
	{
		super(b);
	}

    /**
     * Creates an object representation of the <code>byte</code> argument. 
     *
     * @param   b   a <code>byte</code>.
     */
	public Marshall(byte b)
	{
		super(b);
	}

    /**
     * Creates an object representation of the <code>char</code> argument. 
     *
     * @param   c   a <code>char</code>.
     */
	public Marshall(char c)
	{
		super(c);
	}

    /**
     * Creates an object representation of the <code>short</code> argument. 
     *
     * @param   s   a <code>short</code>.
     */
	public Marshall(short s)
	{
		super(s);
	}

    /**
     * Creates an object representation of the <code>int</code> argument. 
     *
     * @param   i   a <code>int</code>.
     */
	public Marshall(int i)
	{
		super(i);
	}

    /**
     * Creates an object representation of the <code>long</code> argument. 
     *
     * @param   l   a <code>long</code>.
     */
	public Marshall(long l)
	{
		super(l);
	}

    /**
     * Creates an object representation of the <code>float</code> argument. 
     *
     * @param   f   a <code>float</code>.
     */
	public Marshall(float f)
	{
		super(f);
	}

    /**
     * Creates an object representation of the <code>double</code> argument. 
     *
     * @param   d   a <code>double</code>.
     */
	public Marshall(double d)
	{
		super(d);
	}

    /**
     * Creates an object representation of the <code>String</code> argument. 
     *
     * @param   str   a <code>String</code>.
     */
	public Marshall(String str)
	{
		super(str);
	}

    /**
     * Stores a copy of the <code>Object</code> argument. 
     *
     * @param   o   an <code>Object</code>.
     */
	public Marshall(Object o)
	{
		super(o);
	}
	
	/**
     * Stores a copy of the <code>Object</code> argument as the <code>ObjectType</code>
     * passed in. 
	 * 
     * @param   o       an <code>Object</code>.
	 * @param   refType the <code>ObjectType</code> that the object is assigned as.
	 */
	public Marshall(Object o, ObjectType refType)
	{
		super(o, refType);
	}


	/**
	 * This method will check the marshall type passed to see if the instrinsic value's
	 * header byte should be skipped.
	 * @param type
	 * @return
	 */
	public final static boolean skipBinaryHeader(byte type)
	{
		return ((type == IBinaryMarshall.TYPE_CHAR_ARRAY)  ||
				(type == IBinaryMarshall.TYPE_WORD_ARRAY)  ||
				(type == IBinaryMarshall.TYPE_DWORD_ARRAY) ||
				(type == IBinaryMarshall.TYPE_QWORD_ARRAY) ||
				(type == IBinaryMarshall.TYPE_LONG_CHAR_ARRAY));
	}
	

	/**
	 * This function should encode a string buffer.
	 * <p>
	 * NOTE: to encode an array create an ObjectRef array, and then do the 
	 * binaryEncode method on the individual elements
	 */
	public int binaryEncode(ObjectRef buffer, ObjectRef rPos) // VariableToString
	{
		// TODO: Comments
		int ret = binaryEncode(buffer, false);
		rPos.valueOf(rPos.intValue() + ret);
		return ret;
	}

	/**
	 * 
	 * @param buffer
	 * @param skipHeader
	 * @return
	 */
	public int binaryEncode(ObjectRef buffer, boolean skipHeader) // VariableToString
	{
		// TODO: Comments
		int ret = STV_ENCODE_OK;
		ByteArrayOutputStream stream = new ByteArrayOutputStream();
		
		Object ref = getValue();
		if (ref != null)
		{
			if (ref instanceof String)
			{
				return binaryEncodeString(buffer, false);
			}
			else if (ref instanceof Boolean)
			{
				if (false == skipHeader)
				{
					stream.write(TYPE_CHAR);
					ret = 1;
				}
					
				byte[] barray = new byte[1];
				Bits.putBoolean(barray, 0, booleanValue());
				stream.write(barray, 0, barray.length);
				ret += barray.length;
			}
			else if (ref instanceof Byte) 
			{
				if (false == skipHeader)
				{
					stream.write(TYPE_CHAR);
					ret = 1;
				}
					
				stream.write(byteValue());
				ret += 1;
			}
			else if (ref instanceof Character) 
			{
				if (false == skipHeader)
				{
					stream.write(TYPE_WORD);
					ret = 1;
				}
					
				byte[] barray = new byte[2];
				Bits.putChar(barray, 0, charValue());
				stream.write(barray, 0, barray.length);
				ret += barray.length;
			}
			else if (ref instanceof Short)
			{
				if (false == skipHeader)
				{
					stream.write(TYPE_WORD);
					ret = 1;
				}
	
				byte[] barray = new byte[2];
				Bits.putShort(barray, 0, shortValue());
				stream.write(barray, 0, barray.length);
				ret += barray.length;
			}
			else if (ref instanceof Integer) 
			{
				if (false == skipHeader)
				{
					stream.write(TYPE_DWORD);
					ret = 1;
				}
	
				byte[] barray = new byte[4];
				Bits.putInt(barray, 0, intValue());
				stream.write(barray, 0, barray.length);
				ret += barray.length;
			}
			else if (ref instanceof Long) 
			{
				if (false == skipHeader)
				{
					stream.write(TYPE_QWORD);
					ret = 1;
				}
	
				byte[] barray = new byte[8];
				Bits.putLong(barray, 0, longValue());
				stream.write(barray, 0, barray.length);
				ret += barray.length;
			}
			else if (ref instanceof Float) 
			{
				if (false == skipHeader)
				{
					stream.write(TYPE_DWORD);
					ret = 1;
				}

				byte[] barray = new byte[4];
				Bits.putFloat(barray, 0, floatValue());
				stream.write(barray, 0, barray.length);
				ret += barray.length;
			}
			else if (ref instanceof Double)
			{
				if (false == skipHeader)
				{
					stream.write(TYPE_QWORD);
					ret = 1;
				}
				
				byte[] barray = new byte[8];
				Bits.putDouble(barray, 0, doubleValue());
				stream.write(barray, 0, barray.length);
				ret += barray.length;
			}
			else
			{
				ret = STV_ENCODE_VARIABLE_UNKNOWN;
			}
			
		} // if (ref != null)
		
		else
		{
			ret = STV_ENCODE_VARIABLE_UNKNOWN;
		}
		
		if (ret >= STV_ENCODE_OK)
			buffer.valueOf(buffer.stringValue() + stream.toString());

		return ret;
	}

	/**
	 * This function should decode a string buffer
	 * 
	 */
	public int binaryDecode(ObjectRef buffer, ObjectRef rPos) // StringToVariable
	{
		// TODO: Comments
		int ret = binaryDecode(buffer, false);
		rPos.valueOf(rPos.intValue() + ret);
		return ret;
	}
	
	/**
	 * 
	 * @param buffer
	 * @param skipHeader
	 * @return
	 */
	public int binaryDecode(ObjectRef buffer, boolean skipHeader)
	{
		// TODO: Comments
		int ret = STV_DECODE_OK;

		Object ref = getValue();
		if (ref != null)
		{
			try
			{
				// Make sure this is a string
				if (ref != null && buffer != null && ObjectType.STRING.equals(buffer.getType()))
				{
					ByteArrayInputStream stream = new ByteArrayInputStream(buffer.stringValue().getBytes());
					
					int dataType = 0;
					if (false == skipHeader)
					{
						dataType = (byte)stream.read();
						ret = 1;
					}
						
					if (ref instanceof String)
					{
						return binaryDecodeString(buffer);
					}
					else if (ref instanceof Boolean)
					{
						if (dataType != 0 && dataType != TYPE_CHAR)
							return STV_DECODE_VARIABLE_MISMATCH;
						// Do we have what we need?
						if (1 > stream.available())
							return STV_DECODE_DATA_INCOMPLETE;
						
						// Read data
						byte[] barray = new byte[1];
						stream.read(barray, 0, barray.length);
						// TODO: CHECK THIS
						/*ref = */valueOf(Bits.getBoolean(barray, 0));
						ret += barray.length;
					}
					else if (ref instanceof Byte) 
					{
						if (dataType != 0 && dataType != TYPE_CHAR)
							return STV_DECODE_VARIABLE_MISMATCH;
						// Do we have what we need?
						if (1 > stream.available())
							return STV_DECODE_DATA_INCOMPLETE;
						
						// Read data
//						 TODO: CHECK THIS
						/*ref = */valueOf((byte)stream.read());
						ret += 1;
					}
					else if (ref instanceof Character) 
					{
						if (dataType != 0 && dataType != TYPE_WORD)
							return STV_DECODE_VARIABLE_MISMATCH;
						// Do we have what we need?
						if (2 > stream.available())
							return STV_DECODE_DATA_INCOMPLETE;
						
						// Read data
						byte[] barray = new byte[2];
						stream.read(barray, 0, barray.length);
//						 TODO: CHECK THIS
						/*ref = */valueOf(Bits.getChar(barray, 0));
						ret += barray.length;
					}
					else if (ref instanceof Short)
					{
						if (dataType != 0 && dataType != TYPE_WORD)
							return STV_DECODE_VARIABLE_MISMATCH;
						// Do we have what we need?
						if (2 > stream.available())
							return STV_DECODE_DATA_INCOMPLETE;
						
						// Read data
						byte[] barray = new byte[2];
						stream.read(barray, 0, barray.length);
//						 TODO: CHECK THIS
						/*ref = */valueOf(Bits.getShort(barray, 0));
						ret += barray.length;
					}
					else if (ref instanceof Integer) 
					{
						if (dataType != 0 && dataType != TYPE_DWORD)
							return STV_DECODE_VARIABLE_MISMATCH;
						// Do we have what we need?
						if (4 > stream.available())
							return STV_DECODE_DATA_INCOMPLETE;
							
						// Read data
						byte[] barray = new byte[4];
						stream.read(barray, 0, barray.length);
//						 TODO: CHECK THIS
						/*ref = */valueOf(Bits.getInt(barray, 0));
						ret += barray.length;
					}
					else if (ref instanceof Long) 
					{
						if (dataType != 0 && dataType != TYPE_QWORD)
							return STV_DECODE_VARIABLE_MISMATCH;
						// Do we have what we need?
						if (8 > stream.available())
							return STV_DECODE_DATA_INCOMPLETE;
							
						// Read data
						byte[] barray = new byte[8];
						stream.read(barray, 0, barray.length);
//						 TODO: CHECK THIS
						/*ref = */valueOf(Bits.getLong(barray, 0));
						ret += barray.length;
					}
					else if (ref instanceof Float) 
					{
						if (dataType != 0 && dataType != TYPE_DWORD)
							return STV_DECODE_VARIABLE_MISMATCH;
						// Do we have what we need?
						if (4 > stream.available())
							return STV_DECODE_DATA_INCOMPLETE;
							
						// Read data
						byte[] barray = new byte[4];
						stream.read(barray, 0, barray.length);
//						 TODO: CHECK THIS
						/*ref = */valueOf(Bits.getFloat(barray, 0));
						ret += barray.length;
					}
					else if (ref instanceof Double)
					{
						if (dataType != 0 && dataType != TYPE_QWORD)
							return STV_DECODE_VARIABLE_MISMATCH;
						// Do we have what we need?
						if (8 > stream.available())
							return STV_DECODE_DATA_INCOMPLETE;
							
						// Read data
						byte[] barray = new byte[8];
						stream.read(barray, 0, barray.length);
//						 TODO: CHECK THIS
						/*ref = */valueOf(Bits.getDouble(barray, 0));
						ret += barray.length;
					}
					else
					{
						ret = STV_DECODE_VARIABLE_MISMATCH; 
					}
					
				} // if (ref != null && buffer != null && ObjectType.STRING.equals(buffer.getType()))
				else
				{
					ret = STV_DECODE_DATA_INCOMPLETE;
				}
				
			} // try
			
			catch (Exception e)
			{
				ret = STV_DECODE_VARIABLE_MISMATCH; 
			}
			
		} // if (ref != null)
		return ret;
	}
	
	/**
	 * 
	 * @param buffer
	 * @param wideChar
	 * @return
	 */
	public int binaryEncodeString(ObjectRef buffer, boolean wideChar) // VariableToString
	{
		// TODO: Comments
		int ret = STV_ENCODE_OK;

		ByteArrayOutputStream stream = new ByteArrayOutputStream();

		Object ref = getValue();
		if (ref != null)
		{
			if (ref instanceof String)
			{
				String refString = stringValue();
				
				// Get length of string 
				int len = refString.length();
				
				// Let's first see if this can be converted to 8-bit byte
				// array or not (use default encoding)
				try
				{
					byte[] ba1 = refString.getBytes("UTF-8");
					if (!wideChar)
					{
						if (ba1.length < 0x10000)
						{
							len = ba1.length;
							stream.write(TYPE_CHAR_ARRAY);

							// write out the length first
							stream.write((byte)len >> 8);
							stream.write((byte)len);
							
							stream.write(ba1,0,len);			
							ret = len + 3;
						}
						else
						{
							len = ba1.length;
							stream.write(TYPE_LONG_CHAR_ARRAY);
							
							// write out the length first
							byte[] barray = new byte[4];
							Bits.putInt(barray, 0, len);
							stream.write(barray, 0, barray.length);
								
							stream.write(ba1,0,len);			
							ret = len + 5;
						}
					}
					
					// OK, this string is not 8-bit.
					else
					{
						// Ok
						try
						{
							// Try to turn string into UTF-16BE encoding (WORD array)
							// UTF-16 BE startws with 0xFEFF 
							byte[] ba2 = refString.getBytes("UTF-16BE");
							if (ba2.length < 0x10000)
							{
								len = ba2.length;
								stream.write(TYPE_WORD_ARRAY);
								
								// write out the length first
								stream.write((byte)len >> 8);
								stream.write((byte)len);
									
								stream.write(ba2,0,len);
								ret = len + 3;
							}
							else
							{
								// Cannot encode a string longer than 64k
								// Well, I guess I could marshall and array of
								// WORD's but I don't want to...
								ret = STV_ENCODE_VARIABLE_UNKNOWN;
							}
						}
						catch (UnsupportedEncodingException e)
						{
							// Cannot convert to UTF-16BE byte array
							ret = STV_ENCODE_VARIABLE_UNKNOWN;
						}
					}
				}
				catch (UnsupportedEncodingException e)
				{
					// Cannot convert to UTF-8 byte array
					ret = STV_ENCODE_VARIABLE_UNKNOWN;
				}
				
			} // if (ref instanceof String)
			else
			{
				ret = STV_ENCODE_VARIABLE_UNKNOWN;
			}
		}
		else
		{
			ret = STV_ENCODE_VARIABLE_UNKNOWN;
		}
		
		if (ret >= STV_ENCODE_OK)
			buffer.valueOf(buffer.stringValue() + stream.toString());

		return ret;
	}
	
	/**
	 * 
	 * @param buffer
	 * @return
	 */
	public int binaryDecodeString(ObjectRef buffer)
	{
		// TODO: Comments
		int ret = STV_DECODE_OK;
		
		Object ref = getValue();
		if (ref != null)
		{
			if (ref instanceof String)
			{
				ByteArrayInputStream stream = new ByteArrayInputStream(buffer.stringValue().getBytes());
				
				int dataType = stream.read();
				switch ((byte)dataType)
				{
					// Character array - only String supported here				
					case TYPE_CHAR_ARRAY:
					{
						// Do we have what we need?
						if (2 > stream.available())
						{
							ret = STV_DECODE_DATA_INCOMPLETE;
							break;
						}
						
						// Read length (# of elements)
						int len = stream.read();
						len = (len << 8) + stream.read();
						int bytes = len;
						
						// Do we have what we need?
						if (bytes > stream.available())
						{
							ret = STV_DECODE_DATA_INCOMPLETE;
							break;
						}
						
						// Read data
						byte[] data = new byte[bytes];
						bytes = stream.read(data,0,bytes);
						ret = bytes + 3;

						if (ref instanceof String)
						{
							// Try to convert using the basic extended ASCII formatting
							try
							{
								// TODO: CHECK THIS
								valueOf(new String(data, "ISO-8859-1"));
							}
							catch (UnsupportedEncodingException e)
							{
								ret = STV_DECODE_VARIABLE_MISMATCH; 
							}
						}
						else
						{
							ret = STV_DECODE_VARIABLE_MISMATCH; 
						}
						break;
					} // case TYPE_CHAR_ARRAY:
					
					// Word array - only String supported here
					case TYPE_WORD_ARRAY:
					{
						// Do we have what we need?
						if (2 > stream.available())
						{
							ret = STV_DECODE_DATA_INCOMPLETE;
							break;
						}
						
						// Read length (# of elements)
						int len = stream.read();
						len = (len << 8) + stream.read();
						int bytes = len;
							
						// Do we have what we need?
						if (bytes > stream.available())
						{
							ret = STV_DECODE_DATA_INCOMPLETE;
							break;
						}
						
						// Read data
						byte[] data = new byte[bytes];
						bytes = stream.read(data,0,bytes);
						ret = bytes + 3;
						
						// Convert
						if (ref instanceof String)
						{
							// Try to convert using UTF-16BE.
							// Do we need to add the 0xfffe to this stirng 
							// before converting?
							try
							{
								// TODO: CHECK THIS
								valueOf(new String(data, "UTF-16BE"));
							}
							catch (UnsupportedEncodingException e)
							{
								ret = STV_DECODE_VARIABLE_MISMATCH; 
							}
						}
						else
						{
							ret = STV_DECODE_VARIABLE_MISMATCH; 
						}
						break;
					} // case TYPE_WORD_ARRAY:
					
					// Long Char array - String
					case TYPE_LONG_CHAR_ARRAY:
					{
						// Do we have what we need?
						if (4 > stream.available())
						{
							ret = STV_DECODE_DATA_INCOMPLETE;
							break;
						}
						
						// Read length
						byte[] barray = new byte[4];
						stream.read(barray, 0, barray.length);
						int bytes = Bits.getInt(barray, 0);
						
						// Do we have what we need?
						if (bytes > stream.available())
						{
							ret = STV_DECODE_DATA_INCOMPLETE;
							break;
						}
						
						// Read data
						byte[] data = new byte[bytes];
						bytes = stream.read(data,0,bytes);
						ret = bytes + barray.length + 1;

						if (ref instanceof String)
						{
							// Try to convert using the basic extended ASCII formatting
							try
							{
								// TODO: CHECK THIS
								valueOf(new String(data, "ISO-8859-1"));
							}
							catch (UnsupportedEncodingException e)
							{
								ret = STV_DECODE_VARIABLE_MISMATCH; 
							}
						}
						else
						{
							ret = STV_DECODE_VARIABLE_MISMATCH; 
						}
						break;
					} // case TYPE_LONG_CHAR_ARRAY:

				} // switch ((byte)dataType)
				
			} // if (ref instanceof String)
			
		} // if (ref != null)
		return ret;
	}
	
}
