/*
 * @(#)Bits.java	1.2 02/01/03
 *
 * Copyright \251 1994-2002 Sun Microsystems, Inc.  All rights reserved.
 *
 * PROPRIETARY/CONFIDENTIAL
 *
 * Use is subject to license terms.
 *
 */

// Copied from JDK 1.4 for serialization back-port.

package com.amx.duet.tools.lang;

/**
 * Utility methods for packing/unpacking primitive values in/out of byte arrays
 * using big-endian byte ordering.
 */
public class Bits 
{
    /*
     * Methods for unpacking primitive values from byte arrays starting at
     * given offsets.
     */
	
	/**
     * Method for unpacking boolean values from byte arrays starting at
     * given offsets.
	 * 
	 * @param b
	 * @param off
	 * @return
	 */
    public final static boolean getBoolean(byte[] b, int off)
    {
		return b[off] != 0;
    }
    
    /**
     * Method for unpacking character values from byte arrays starting at
     * given offsets.
     * 
     * @param b
     * @param off
     * @return
     */
    public final static char getChar(byte[] b, int off)
    {
		return (char) (((b[off + 1] & 0xFF) << 0) + 
			       ((b[off + 0] & 0xFF) << 8));
    }
    
    /**
     * Method for unpacking short values from byte arrays starting at
     * given offsets.
     * 
     * @param b
     * @param off
     * @return
     */
    public final static short getShort(byte[] b, int off) 
    {
		return (short) (((b[off + 1] & 0xFF) << 0) + 
				((b[off + 0] & 0xFF) << 8));
    }
    
    /**
     * Method for unpacking integer values from byte arrays starting at
     * given offsets.
     * 
     * @param b
     * @param off
     * @return
     */
    public final static int getInt(byte[] b, int off) 
    {
		return ((b[off + 3] & 0xFF) << 0) +
		       ((b[off + 2] & 0xFF) << 8) +
		       ((b[off + 1] & 0xFF) << 16) +
		       ((b[off + 0] & 0xFF) << 24);
    }
    
    /**
     * Method for unpacking float values from byte arrays starting at
     * given offsets.
     * 
     * @param b
     * @param off
     * @return
     */
    public final static float getFloat(byte[] b, int off) 
    {
		int i = ((b[off + 3] & 0xFF) << 0) +
			((b[off + 2] & 0xFF) << 8) +
			((b[off + 1] & 0xFF) << 16) +
			((b[off + 0] & 0xFF) << 24);
		return Float.intBitsToFloat(i);
    }
    
    /**
     * Method for unpacking long values from byte arrays starting at
     * given offsets.
     * 
     * @param b
     * @param off
     * @return
     */
    public final static long getLong(byte[] b, int off) 
    {
		return ((b[off + 7] & 0xFFL) << 0) +
		       ((b[off + 6] & 0xFFL) << 8) +
		       ((b[off + 5] & 0xFFL) << 16) +
		       ((b[off + 4] & 0xFFL) << 24) +
		       ((b[off + 3] & 0xFFL) << 32) +
		       ((b[off + 2] & 0xFFL) << 40) +
		       ((b[off + 1] & 0xFFL) << 48) +
		       ((b[off + 0] & 0xFFL) << 56);
    }

    /**
     * Method for unpacking double values from byte arrays starting at
     * given offsets.
     * 
     * @param b
     * @param off
     * @return
     */
    public final static double getDouble(byte[] b, int off) 
    {
		long j = ((b[off + 7] & 0xFFL) << 0) +
			 ((b[off + 6] & 0xFFL) << 8) +
			 ((b[off + 5] & 0xFFL) << 16) +
			 ((b[off + 4] & 0xFFL) << 24) +
			 ((b[off + 3] & 0xFFL) << 32) +
			 ((b[off + 2] & 0xFFL) << 40) +
			 ((b[off + 1] & 0xFFL) << 48) +
			 ((b[off + 0] & 0xFFL) << 56);
		return Double.longBitsToDouble(j);
    }
    
    /*
     * Methods for packing primitive values into byte arrays starting at given
     * offsets.
     */

    /**
     * Method for packing boolean values into byte arrays starting at given
     * offsets.
     * 
     * @param b
     * @param off
     * @param val
     */
    public final static void putBoolean(byte[] b, int off, boolean val) 
    {
		b[off] = (byte) (val ? 1 : 0);
    }

    /**
     * Method for packing character values into byte arrays starting at given
     * offsets.
     * 
     * @param b
     * @param off
     * @param val
     */
    public final static void putChar(byte[] b, int off, char val) 
    {
		b[off + 1] = (byte) (val >>> 0);
		b[off + 0] = (byte) (val >>> 8);
    }

    /**
     * Method for packing short values into byte arrays starting at given
     * offsets.
     * 
     * @param b
     * @param off
     * @param val
     */
    public final static void putShort(byte[] b, int off, short val) 
    {
		b[off + 1] = (byte) (val >>> 0);
		b[off + 0] = (byte) (val >>> 8);
    }

    /**
     * Method for packing integer values into byte arrays starting at given
     * offsets.
     * 
     * @param b
     * @param off
     * @param val
     */
    public final static void putInt(byte[] b, int off, int val) 
    {
		b[off + 3] = (byte) (val >>> 0);
		b[off + 2] = (byte) (val >>> 8);
		b[off + 1] = (byte) (val >>> 16);
		b[off + 0] = (byte) (val >>> 24);
    }

    /**
     * Method for packing float values into byte arrays starting at given
     * offsets.
     * 
     * @param b
     * @param off
     * @param val
     */
    public final static void putFloat(byte[] b, int off, float val) 
    {
		int i = Float.floatToIntBits(val);
		b[off + 3] = (byte) (i >>> 0);
		b[off + 2] = (byte) (i >>> 8);
		b[off + 1] = (byte) (i >>> 16);
		b[off + 0] = (byte) (i >>> 24);
    }

    /**
     * Method for packing long values into byte arrays starting at given
     * offsets.
     * 
     * @param b
     * @param off
     * @param val
     */
    public final static void putLong(byte[] b, int off, long val) 
    {
		b[off + 7] = (byte) (val >>> 0);
		b[off + 6] = (byte) (val >>> 8);
		b[off + 5] = (byte) (val >>> 16);
		b[off + 4] = (byte) (val >>> 24);
		b[off + 3] = (byte) (val >>> 32);
		b[off + 2] = (byte) (val >>> 40);
		b[off + 1] = (byte) (val >>> 48);
		b[off + 0] = (byte) (val >>> 56);
    }

    /**
     * Method for packing double values into byte arrays starting at given
     * offsets.
     * 
     * @param b
     * @param off
     * @param val
     */
    public final static void putDouble(byte[] b, int off, double val) 
    {
		long j = Double.doubleToLongBits(val);
		b[off + 7] = (byte) (j >>> 0);
		b[off + 6] = (byte) (j >>> 8);
		b[off + 5] = (byte) (j >>> 16);
		b[off + 4] = (byte) (j >>> 24);
		b[off + 3] = (byte) (j >>> 32);
		b[off + 2] = (byte) (j >>> 40);
		b[off + 1] = (byte) (j >>> 48);
		b[off + 0] = (byte) (j >>> 56);
    }

    /**
     * Checks to see if the bit position in an 8 bit value is set. Position
     * starts at 0 (the lowest bit) and ends a 7 (the 8th bit).
     * 
     * @param value 8 bit value
     * @param pos 0-7 (bytes 1-8 respectively)
     * @return true if the bit is set; false otherwise.
     */
	public final static boolean isBitSet(int value, int pos)
	{
		return ((((value & 0xff) >> pos) & 0x01) > 0);
	}
	
	/**
	 * Changes the order of a byte array for endianness.
	 * 
	 * @param b byte array to flip.
	 * @return the byte array in reverse order.
	 */
	public final static byte[] flipBytes(byte[] b)
	{
		byte bReturn[] = new byte[b.length];
		for (int i=b.length-1; i>=0;i--)
			bReturn[b.length-(i+1)] = b[i];
		return bReturn;
	}
}
