/*
 * ClassName.java
 * 
 * Created on Jul 7, 2006
 * Created by VGonzales
 *
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.lang;

import java.util.StringTokenizer;

/**
 * Utility class for parsing out the class name for a given class.
 * 
 * @version 1.0.0
 */
public class ClassName
{
	/**
	 * Gets the class name for the Object passed in using java's Class reflection
	 * 
	 * @param rStr Object whose name is to be retrieved
	 * @return String containing only the Class name without the package details
	 */
	public final static String getClassName(Object rStr)
	{
		String className = "";
		if (rStr != null)
			className = getClassName(rStr.getClass().getName());
		else
			throw new NullPointerException("Class must be initialized");
			
		return className;
	}

	/**
	 * Gets the class name for the Class passed in using java's Class reflection
	 * 
	 * @param cls Class whose name is to be retrieved
	 * @return String containing only the Class name without the package details
	 */
	public final static String getClassName(Class cls)
	{
		String className = "";
		if (cls != null)
			className = getClassName(cls.getName());
		else
			throw new NullPointerException("Class must be initialized");
			
		return className;
	}

	/**
	 * Parses a dot delimited String for the last separated element in the string. In 
	 * this case we are trying to retrieve the Class name from it's entire package name
	 * 
	 * @param name dot delimited String containing the name to be retrieved
	 * @return The last String in a dot separated string
	 */
	public final static String getClassName(String name)
	{
		String className = "";
		if (name != null && name.length() > 0)
		{
			String []values = parse(name, ".");
			className = values[values.length-1];
		}
		else
			throw new NullPointerException("Class must be initialized");
			
		return className;
	}

//    /**
//     * Return the name of the routine that called getCurrentMethodName
//     * 
//     * @return String containing the current method name.
//     */
//    public final static String getCurrentMethodName() 
//    {
//        return trace("getCurrentMethodName").getMethodName();
//    }
//    
//	/**
//	 * @deprecated see getCurrentMethod()
//	 * 
//	 * @param cls
//	 * @return
//	 */
//	public final static Method[] getCurrentMethod(Object obj)
//	{
//		Vector meths = new Vector();
//		StackTraceElement el = trace("getCurrentMethod");
//		try
//		{
//			if (el != null)
//			{
//				String name = el.getMethodName();
//				Method[] methods = obj.getClass().getDeclaredMethods();
//				for (int x = 0; x < methods.length; x++)
//				{
//					if (name.equals(methods[x].getName()))
//						meths.add(methods[x]);
//				}
//			}
//		}
//		catch (Exception e)
//		{
//			
//		}
//		
//		Method[] temp = new Method[meths.size()];
//		return (Method[]) meths.toArray(temp);
//	}
//	
//	/**
//	 * Get a list of all the methods that match the current method name.
//	 * 
//	 * @return Method[]
//	 */
//	public final static Method[] getCurrentMethod()
//	{
//		Vector meths = new Vector();
//		StackTraceElement el = trace("getCurrentMethod");
//		try
//		{
//			if (el != null)
//			{
//				String name = el.getMethodName();
//		        
//				Class cls = Class.forName(el.getClassName());
//				Method[] methods = cls.getDeclaredMethods();
//				for (int x = 0; x < methods.length; x++)
//				{
//					if (name.equals(methods[x].getName()))
//						meths.add(methods[x]);
//				}
//			}
//		}
//		catch (Exception e) {}
//		
//		Method[] methods = new Method[meths.size()];
//		return (Method[]) meths.toArray(methods);
//	}
//	
//	 private static StackTraceElement trace(String exclude) 
//	 {
//		   boolean doNext = false;
//		   StackTraceElement e[] = (new Throwable()).getStackTrace();
//		   for (int i = 0; i < e.length; i++) 
//		   {
//		       if (doNext) 
//		          return e[i];
//		       doNext = e[i].getMethodName().equals(exclude);
//		   }
//		   
//		   return null;
//	 }
	 
	/*
	 * 
	 * @param data
	 * @param delim
	 * @return
	 */
	private final static String[] parse(String data, String delim)
	{
		StringTokenizer tokenizer = new StringTokenizer(data, delim);
		String value[] = new String[tokenizer.countTokens()];
		int count = 0;
		
		while (tokenizer.hasMoreTokens()) 
		{
			value[count++] = tokenizer.nextToken();
		}
		return value;
	}
}
