/*
 * @author VGonzales
 * Created on Apr 19, 2005
 * 
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.lang;

/**
 * This class can be used for writing out diagnostic and error messages to the 
 * output stream. A wrapper for that System.out.println() method that formats 
 * the output with the name of the originating class.
 * 
 * @version 1.0.0
 */
public class Logger
{
	/**
	 * Similar the Service.DEBUG
	 */
	public static final int LOG_DEBUG = 4;
	/**
	 * Similar the Service.INFO
	 */
	public static final int LOG_INFO = 3;
	/**
	 * Similar the Service.WARNING
	 */
	public static final int LOG_WARNING = 2;
	/**
	 * Similar the Service.ERROR
	 */
	public static final int LOG_ERROR = 1;
	
	private boolean bDebug = false;
	private String className = "";
	
	public Logger()
	{
		className = ClassName.getClassName(this);
	}
	
	/**
	 * Use this to display diagnostic messages, if you are not extending Diag
	 * 
	 * @param obj method caller (usually the <strong>this</strong> member)
	 * @param str data to be displayed
	 */
	public final static void diag(Object obj, String str)
	{
		if (obj != null)
		{
			System.out.println(ClassName.getClassName(obj) + ": " + str);
		}
		else
		{
			System.out.println("Diag: " + str);
		}
	}

	/**
	 * Use this to display diagnostic messages, if you are not extending Diag
	 * 
	 * @param cls Class calling the method (usually the <strong>.class</strong> member)
	 * @param str data to be displayed
	 */
	public final static void diag(Class cls, String str)
	{
		System.out.println(ClassName.getClassName(cls) + ": " + str);
	}
	
	/**
	 * Use this to display diagnostic messages, if you are not extending Diag
	 * 
	 * @param sClassName name of the class to prepend to log message
	 * @param str data to be displayed
	 */
	public final static void diag(String sClassName, String str)
	{
		System.out.println(sClassName + ": " + str);
	}

	/**
	 * Use this to display error messages, if you are not extending Diag
	 * 
	 * @param obj method caller (usually the <strong>this</strong> member)
	 * @param str data to be displayed
	 */
	public final static void error(Object obj, String str)
	{
		if (obj != null)
		{
			System.out.println("ERROR: " + ClassName.getClassName(obj) + ": " + str);
		}
		else
		{
			System.out.println("ERROR: Diag: " + str);
		}
	}

	/**
	 * Use this to display error messages, if you are not extending Diag
	 * 
	 * @param cls Class calling the method (usually the <strong>.class</strong> member)
	 * @param str data to be displayed
	 */
	public final static void error(Class cls, String str)
	{
		System.out.println("ERROR: " + ClassName.getClassName(cls) + ": " + str);
	}

	/**
	 * Use this to display error messages, if you are not extending Diag
	 * 
	 * @param sClassName name of the class to prepend to error message
	 * @param str data to be displayed
	 */
	public final static void error(String sClassName, String str)
	{
		System.out.println("ERROR: " + sClassName + ": " + str);
	}

	/**
	 * Use this to display diagnostic messages, if you are extending Diag
	 * 
	 * @param obj calling object
	 * @param str data to be displayed
	 */
	public void diag(String str)
	{
		System.out.println(className + ": " + str);
	}

	/**
	 * Use this to display error messages, if you are extending Diag
	 * 
	 * @param obj calling object
	 * @param str data to be displayed
	 */
	public void error(String str)
	{
		System.out.println("ERROR: " + className + ": " + str);
	}

	/**
	 * This will only display messages if debugging is enabled with the <code>enableDebug()</code>
	 * method
	 * 
	 * @param str str data to be displayed
	 */
	public void debug(String str)
	{
		if (bDebug)
			diag(str);
	}
	
	/**
	 * Enable debug messages to be displayed by setting the debug state flag. Can also be used by
	 * derived classes as an internal debug state flag.
	 *  
	 * @param bState true if debugging should be enable; false otherwise.
	 */
	public final void enableDebug(boolean bState)
	{
		bDebug = bState;
	}
	
	/**
	 * Check to see if the debug state flag has been set to true or false.
	 * 
	 * @return true if debugging is enabled; false otherwise.
	 */
	public final boolean isDebugEnabled()
	{
		return bDebug;
	}

}
