/*
 * @author VGonzales
 * Created on May 12, 2005
 * 
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.lang;

import java.util.Iterator;


/**
 * An object reference class used to convert objects from one value or data type 
 * to another. This class was created to handle parameter pass by reference 
 * values.
 *  
 * @version 1.0.0
 * @since AMXTools 1.0.0
 */
public class ObjectRef// implements IBinaryMarshall
{
	/**
	 * 
	 */
	private Object ref = null;
	
	/**
	 * 
	 */
	private ObjectType type = ObjectType.INVALID;
	
    /**
     * Creates an object representation of the <code>boolean</code> argument. 
     *
     * @param   b   a <code>boolean</code>.
     */
	public ObjectRef(boolean b)
	{
		valueOf(b);
	}

    /**
     * Creates an object representation of the <code>byte</code> argument. 
     *
     * @param   b   a <code>byte</code>.
     */
	public ObjectRef(byte b)
	{
		valueOf(b);
	}

    /**
     * Creates an object representation of the <code>char</code> argument. 
     *
     * @param   c   a <code>char</code>.
     */
	public ObjectRef(char c)
	{
		valueOf(c);
	}

    /**
     * Creates an object representation of the <code>short</code> argument. 
     *
     * @param   s   a <code>short</code>.
     */
	public ObjectRef(short s)
	{
		valueOf(s);
	}

    /**
     * Creates an object representation of the <code>int</code> argument. 
     *
     * @param   i   a <code>int</code>.
     */
	public ObjectRef(int i)
	{
		valueOf(i);
	}

    /**
     * Creates an object representation of the <code>long</code> argument. 
     *
     * @param   l   a <code>long</code>.
     */
	public ObjectRef(long l)
	{
		valueOf(l);
	}

    /**
     * Creates an object representation of the <code>float</code> argument. 
     *
     * @param   f   a <code>float</code>.
     */
	public ObjectRef(float f)
	{
		valueOf(f);
	}

    /**
     * Creates an object representation of the <code>double</code> argument. 
     *
     * @param   d   a <code>double</code>.
     */
	public ObjectRef(double d)
	{
		valueOf(d);
	}

    /**
     * Creates an object representation of the <code>String</code> argument. 
     *
     * @param   str   a <code>String</code>.
     */
	public ObjectRef(String str)
	{
		valueOf(str);
	}

    /**
     * Stores a copy of the <code>Object</code> argument. 
     *
     * @param   o   an <code>Object</code>.
     */
	public ObjectRef(Object o)
	{
		if (o != null)
			valueOf(o);
		else
			throw new NullPointerException();
	}
	
	/**
     * Stores a copy of the <code>Object</code> argument as the <code>ObjectType</code>
     * passed in. 
	 * 
     * @param   o       an <code>Object</code>.
	 * @param   refType the <code>ObjectType</code> that the object is assigned as.
	 */
	public ObjectRef(Object o, ObjectType refType)
	{
		if (o != null && refType != null)
			valueOf(o, refType);
		else
			throw new NullPointerException();
	}
	
	/**
	 * Creates an empty object representation of the type of <code>ObjectType</code> 
	 * passed in.
	 * 
	 * @param   refType the <code>ObjectType</code> that the object is assigned as.
	 * @exception UnsupportedOperationException if the object type passed in is not 
	 * supported.
	 */
	public ObjectRef(ObjectType refType)
	{
		if (refType != null)
		{
			type = refType;
			if (ObjectType.BOOL.equals(refType))
				ref = new Boolean(false);
			else if (ObjectType.BYTE.equals(refType))
				ref = new Byte((byte)0);
			else if (ObjectType.CHAR.equals(refType))
				ref = new Character((char)0);
			else if (ObjectType.SHORT.equals(refType))
				ref = new Short((short)0);
			else if (ObjectType.INTEGER.equals(refType))
				ref = new Integer(0);
			else if (ObjectType.LONG.equals(refType))
				ref = new Long(0);
			else if (ObjectType.FLOAT.equals(refType))
				ref = new Float(0);
			else if (ObjectType.DOUBLE.equals(refType))
				ref = new Double(0);
			else if (ObjectType.STRING.equals(refType))
				ref = new String("");
			else
				throw new UnsupportedOperationException(refType + " object creation is not supported by this construction method");
		}
		else
		{
			throw new NullPointerException();
		}
	}
	
    /**
     * Stores and returns the object representation of the <code>boolean</code> argument. 
     *
     * @param   b   a <code>boolean</code>.
     * @return  The <code>boolean</code> value created as a <code>Boolean</code> 
     * object and cast to an <code>Object</code>.
     */
	public Object valueOf(boolean b)
	{
		ref = new Boolean(b);
		type = ObjectType.BOOL;
		return ref;
	}

    /**
     * Stores and returns the object representation of the <code>byte</code> argument. 
     *
     * @param   b   a <code>byte</code>.
     * @return  The <code>byte</code> value created as a <code>Byte</code> object 
     * and cast to an <code>Object</code>.
     */
	public Object valueOf(byte b)
	{
		ref = new Byte(b);
		type = ObjectType.BYTE;
		return ref;
	}

    /**
     * Stores and returns the object representation of the <code>char</code> argument. 
     *
     * @param   c   a <code>char</code>.
     * @return  The <code>char</code> value created as a <code>Character</code>  
     * object and cast to an <code>Object</code>.
     */
	public Object valueOf(char c)
	{
		ref = new Character(c);
		type = ObjectType.CHAR;
		return ref;
	}

    /**
     * Stores and returns the object representation of the <code>short</code> argument. 
     *
     * @param   s   a <code>short</code>.
     * @return  The <code>short</code> value created as a <code>Short</code>  
     * object and cast to an <code>Object</code>.
     */
	public Object valueOf(short s)
	{
		ref = new Short(s);
		type = ObjectType.SHORT;
		return ref;
	}

    /**
     * Stores and returns the object representation of the <code>int</code> argument. 
     *
     * @param   i   an <code>int</code>.
     * @return  The <code>int</code> value created as a <code>Integer</code>  
     * object and cast to an <code>Object</code>.
     */
	public Object valueOf(int i)
	{
		ref = new Integer(i);
		type = ObjectType.INTEGER;
		return ref;
	}

    /**
     * Stores and returns the object representation of the <code>long</code> argument. 
     *
     * @param   l   a <code>long</code>.
     * @return  The <code>long</code> value created as a <code>Long</code>  
     * object and cast to an <code>Object</code>.
     */
	public Object valueOf(long l)
	{
		ref = new Long(l);
		type = ObjectType.LONG;
		return ref;
	}

    /**
     * Stores and returns the object representation of the <code>floag</code> argument. 
     *
     * @param   f   a <code>float</code>.
     * @return  The <code>float</code> value created as a <code>Float</code>  
     * object and cast to an <code>Object</code>.
     */
	public Object valueOf(float f)
	{
		ref = new Float(f);
		type = ObjectType.FLOAT;
		return ref;
	}

    /**
     * Stores and returns the object representation of the <code>double</code> argument. 
     *
     * @param   d   a <code>double</code>.
     * @return  The <code>double</code> value created as a <code>Double</code>  
     * object and cast to an <code>Object</code>.
     */
	public Object valueOf(double d)
	{
		ref = new Double(d);
		type = ObjectType.DOUBLE;
		return ref;
	}

    /**
     * Stores and returns the object representation of the <code>String</code> argument. 
     *
     * @param   str a <code>String</code>.
     * @return  The <code>String</code> value cast to an <code>Object</code>.
     */
	public Object valueOf(String str)
	{
		ref = str;
		type = ObjectType.STRING;
		return ref;
	}

    /**
     * Stores and returns the <code>Object</code> argument. 
     *
     * @param   o an <code>Object</code>.
     * @return  The <code>Object</code> argument passed it.
     */
	public Object valueOf(Object o)
	{
		ref = o;
		type = ObjectType.OBJECT;
		return ref;
	}
	
    /**
     * Stores and returns the <code>Object</code> argument as the <code>ObjectType</code>
     * passed in. 
     *
     * @param   o an <code>Object</code>.
     * @param   refType 
     * @return  The <code>Object</code> argument passed it.
	 * @exception UnsupportedOperationException if the object type passed in is not 
	 * supported.
     */
	public Object valueOf(Object o, ObjectType refType)
	{
		ObjectType ot = ObjectType.INVALID;
		try
		{
			if (refType != null)
			{
				Iterator iter = ObjectType.VALUES.iterator();
				while(iter.hasNext())
				{
					ObjectType temp = (ObjectType)iter.next();
					if (refType.equals(temp))
					{
						ot = temp;
						break;
					}
				}
				
			}
		}
		catch (Exception e) {}
		
		if (ot.equals(ObjectType.INVALID))
			throw new UnsupportedOperationException(refType + " object creation for this type is not supported");
		
		ref = o;
		type = refType;
		return ref;
	}
	
	/**
	 * Returns the <code>boolean</code> value stored within this object.
	 * 
	 * @return If this object was created from a <code>boolean</code> value, then 
	 * that value will be returned. Otherwise, <code>false</code> will be returned. 
	 */
	public boolean booleanValue()
	{
		if (ref != null && ref instanceof Boolean)
			return ((Boolean)ref).booleanValue();
		return false;
	}

	/**
	 * Returns the <code>byte</code> value stored within this object.
	 * 
	 * @return If this object was created from a <code>byte</code> value, then 
	 * that value will be returned. Otherwise, <code>0</code> will be returned. 
	 */
	public byte byteValue()
	{
		if (ref != null && ref instanceof Byte)
			return ((Byte)ref).byteValue();
		return 0;
	}

	/**
	 * Returns the <code>char</code> value stored within this object.
	 * 
	 * @return If this object was created from a <code>char</code> value, then 
	 * that value will be returned. Otherwise, <code>0</code> will be returned. 
	 */
	public char charValue()
	{
		if (ref != null && ref instanceof Character)
			return ((Character)ref).charValue();
		return 0;
	}

	/**
	 * Returns the <code>short</code> value stored within this object.
	 * 
	 * @return If this object was created from a <code>short</code> value, then 
	 * that value will be returned. Otherwise, <code>0</code> will be returned. 
	 */
	public short shortValue()
	{
		if (ref != null && ref instanceof Short)
			return ((Short)ref).shortValue();
		return 0;
	}

	/**
	 * Returns the <code>int</code> value stored within this object.
	 * 
	 * @return If this object was created from a <code>int</code> value, then 
	 * that value will be returned. Otherwise, <code>0</code> will be returned. 
	 */
	public int intValue()
	{
		if (ref != null && ref instanceof Integer)
			return ((Integer)ref).intValue();
		return 0;
	}

	/**
	 * Returns the <code>long</code> value stored within this object.
	 * 
	 * @return If this object was created from a <code>long</code> value, then 
	 * that value will be returned. Otherwise, <code>0</code> will be returned. 
	 */
	public long longValue()
	{
		if (ref != null && ref instanceof Long)
			return ((Long)ref).longValue();
		return 0;
	}

	/**
	 * Returns the <code>float</code> value stored within this object.
	 * 
	 * @return If this object was created from a <code>float</code> value, then 
	 * that value will be returned. Otherwise, <code>0</code> will be returned. 
	 */
	public float floatValue()
	{
		if (ref != null && ref instanceof Float)
			return ((Float)ref).floatValue();
		return 0;
	}

	/**
	 * Returns the <code>double</code> value stored within this object.
	 * 
	 * @return If this object was created from a <code>double</code> value, then 
	 * that value will be returned. Otherwise, <code>0</code> will be returned. 
	 */
	public double doubleValue()
	{
		if (ref != null && ref instanceof Double)
			return ((Double)ref).doubleValue();
		return 0;
	}
	
	/**
	 * Returns the <code>String</code> value stored within this object.
	 * 
	 * @return If this object was created from a <code>String</code> value, then 
	 * that value will be returned. Otherwise, an empty string will be returned. 
	 */
	public String stringValue()
	{
		if (ref != null && ref instanceof String)
			return (String)ref;
		return "";
	}
	
	/**
	 * Returns the object representation of the value stored in this object.
	 * 
     * @return  The <code>Object</code> stored in this object.
	 */
	public Object getValue()
	{
		return ref;
	}
	
	/**
	 * Returns the object type for the currently stored object.
	 * 
	 * @return <code>ObjectType</code> representing the type of object stored.
	 */
	public ObjectType getType()
	{
		// TODO: Comments
		return type;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj)
	{
		return ref.equals(obj);
	}
	
	/*
	 *  (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		return ref.toString();
	}
	
	/*
	 * This method can be used to retrieve only valid numeric values up until
	 * an invalid character is found.  In other words it will build a string of
	 * numbers (negative included) until a non numeric is reach and it will return
	 * only that part of the string.  So if you had a float value that you wanted
	 * to convert to an int value the method will accumulate the numeric values up
	 * until it sees the "." character, giving you only the whole number. 
	 * 
	 * @param value String containing numeric value to parse for
	 * @return String containing a valid numeric value including signed values.
	 */
	private String getNumeric(String value)
	{
		StringBuffer sbString = new StringBuffer();
		
		// find first non-numerical character and filter out all other characters 
		for (int i = 0; i < value.length(); i++)
		{
			char c = value.charAt(i);
			if ((c == '-') || ((c >= '0') && (c <= '9')))
				sbString.append(c);
			else
				break;
		}
		
		return sbString.toString();
	}
	
	/**
	 * Converts the stored value to a <code>boolean</code> representation and returns
	 * the new value. 
	 * 
	 * @return The converted <code>boolean</code> value. If the stored value is equal to 
	 * "true" or a value other than 0, the return value will be <code>true</code>. 
	 * Otherwise, the return value should be false.
	 * @exception ClassCastException if the stored value could not be converted to a known type.
	 */
	public boolean toBoolean()
	{
		if (ref != null)
		{
			if (ref instanceof String)
			{
				if ((!((String)ref).equals("0")) && (((String)ref).equals("true")))
					return true;
				return (Boolean.valueOf((String)ref)).booleanValue();
			}
			else if (ref instanceof Boolean)
				return booleanValue();
			else if (ref instanceof Byte) 
				return (byteValue() != 0);
			else if (ref instanceof Character) 
				return (charValue() != 0);
			else if (ref instanceof Short)
				return (shortValue() != 0);
			else if (ref instanceof Integer) 
				return (intValue() != 0);
			else if (ref instanceof Long) 
				return (longValue() != 0);
			else if (ref instanceof Float) 
				return (floatValue() != 0);
			else if (ref instanceof Double)
				return (doubleValue() != 0);
			else
				throw new ClassCastException("Object reference could not be casted");
		}
		
		return false;
	}
	
	/**
	 * Converts the stored value to a <code>byte</code> representation and returns
	 * the new value. 
	 * 
	 * @return The converted <code>byte</code> value. If the stored value is equal to 
	 * "true", the return value will be <code>1</code>. If the value could not be 
	 * converted the return value will be <code>0</code>. Otherwise, the return value 
	 * should be a value between <code>Byte.MIN_VALUE</code> and 
	 * <code>Byte.MAX_VALUE</code>.
	 * @exception ClassCastException if the stored value could not be converted to a known type.
	 */
	public byte toByte()
	{
		if (ref != null)
		{
			if (ref instanceof String)
			{
				String val = getNumeric((String)ref);
				if (val.equals("true"))
					return (byte)1;
				return Byte.parseByte(val);
			}
			else if (ref instanceof Boolean)
				return (booleanValue() ? (byte)1 : 0);
			else if (ref instanceof Byte) 
				return byteValue();
			else if (ref instanceof Character) 
				return (byte)charValue();
			else if (ref instanceof Short)
				return (byte)shortValue();
			else if (ref instanceof Integer) 
				return (byte)intValue();
			else if (ref instanceof Long) 
				return (byte)longValue();
			else if (ref instanceof Float) 
				return (byte)floatValue();
			else if (ref instanceof Double)
				return (byte)doubleValue();
			else
				throw new ClassCastException("Object reference could not be casted");
		}
		
		return 0;
	}

	/**
	 * Converts the stored value to a <code>char</code> representation and returns
	 * the new value. 
	 * 
	 * @return The converted <code>char</code> value. If the stored value is equal to 
	 * "true", the return value will be <code>1</code>. If the value could not be 
	 * converted the return value will be <code>0</code>. Otherwise, the return value 
	 * should be a value between <code>Character.MIN_VALUE</code> and 
	 * <code>Character.MAX_VALUE</code>.
	 * @exception ClassCastException if the stored value could not be converted to a known type.
	 */
	public char toChar()
	{
		if (ref != null)
		{
			if (ref instanceof String)
			{
				String val = getNumeric((String)ref);
				if (val.equals("true"))
					return (char)1;
				return (char)Integer.parseInt(val);
			}
			else if (ref instanceof Boolean)
				return (booleanValue() ? (char)1 : 0);
			else if (ref instanceof Byte) 
				return (char)byteValue();
			else if (ref instanceof Character) 
				return charValue();
			else if (ref instanceof Short)
				return (char)shortValue();
			else if (ref instanceof Integer) 
				return (char)intValue();
			else if (ref instanceof Long) 
				return (char)longValue();
			else if (ref instanceof Float) 
				return (char)floatValue();
			else if (ref instanceof Double)
				return (char)doubleValue();
			else
				throw new ClassCastException("Object reference could not be casted");
		}
		
		return 0;
	}

	/**
	 * Converts the stored value to a <code>short</code> representation and returns
	 * the new value. 
	 * 
	 * @return The converted <code>short</code> value. If the stored value is equal to 
	 * "true", the return value will be <code>1</code>. If the value could not be 
	 * converted the return value will be <code>0</code>. Otherwise, the return value 
	 * should be a value between <code>Short.MIN_VALUE</code> and 
	 * <code>Short.MAX_VALUE</code>.
	 * @exception ClassCastException if the stored value could not be converted to a known type.
	 */
	public short toShort()
	{
		if (ref != null)
		{
			if (ref instanceof String)
			{
				String val = getNumeric((String)ref);
				if (val.equals("true"))
					return (short)1;
				return Short.parseShort(val);
			}
			else if (ref instanceof Boolean)
				return (booleanValue() ? (short)1 : 0);
			else if (ref instanceof Byte) 
				return byteValue();
			else if (ref instanceof Character) 
				return (short)charValue();
			else if (ref instanceof Short)
				return shortValue();
			else if (ref instanceof Integer) 
				return (short)intValue();
			else if (ref instanceof Long) 
				return (short)longValue();
			else if (ref instanceof Float) 
				return (short)floatValue();
			else if (ref instanceof Double)
				return (short)doubleValue();
			else
				throw new ClassCastException("Object reference could not be casted");
		}
		
		return 0;
	}
	
	/**
	 * Converts the stored value to a <code>int</code> representation and returns
	 * the new value. 
	 * 
	 * @return The converted <code>int</code> value. If the stored value is equal to 
	 * "true", the return value will be <code>1</code>. If the value could not be 
	 * converted the return value will be <code>0</code>. Otherwise, the return value 
	 * should be a value between <code>Integer.MIN_VALUE</code> and 
	 * <code>Integer.MAX_VALUE</code>.
	 * @exception ClassCastException if the stored value could not be converted to a known type.
	 */
	public int toInt()
	{
		if (ref != null)
		{
			if (ref instanceof String)
			{
				String val = getNumeric((String)ref);
				if (val.equals("true"))
					return 1;
				return Integer.parseInt(val);
			}
			else if (ref instanceof Boolean)
				return (booleanValue() ? (int)1 : 0);
			else if (ref instanceof Byte) 
				return byteValue();
			else if (ref instanceof Character) 
				return charValue();
			else if (ref instanceof Short)
				return shortValue();
			else if (ref instanceof Integer) 
				return intValue();
			else if (ref instanceof Long) 
				return (int)longValue();
			else if (ref instanceof Float) 
				return (int)floatValue();
			else if (ref instanceof Double)
				return (int)doubleValue();
			else
				throw new ClassCastException("Object reference could not be casted");
		}
		
		return 0;
	}

	/**
	 * Converts the stored value to a <code>long</code> representation and returns
	 * the new value. 
	 * 
	 * @return The converted <code>long</code> value. If the stored value is equal to 
	 * "true", the return value will be <code>1</code>. If the value could not be 
	 * converted the return value will be <code>0</code>. Otherwise, the return value 
	 * should be a value between <code>Long.MIN_VALUE</code> and 
	 * <code>Long.MAX_VALUE</code>.
	 * @exception ClassCastException if the stored value could not be converted to a known type.
	 */
	public long toLong()
	{
		if (ref != null)
		{
			if (ref instanceof String)
			{
				String val = getNumeric((String)ref);
				if (val.equals("true"))
					return 1l;
				return Long.parseLong(val);
			}
			else if (ref instanceof Boolean)
				return (booleanValue() ? (long)1 : 0);
			else if (ref instanceof Byte) 
				return byteValue();
			else if (ref instanceof Character) 
				return charValue();
			else if (ref instanceof Short)
				return shortValue();
			else if (ref instanceof Integer) 
				return intValue();
			else if (ref instanceof Long) 
				return longValue();
			else if (ref instanceof Float) 
				return (long)floatValue();
			else if (ref instanceof Double)
				return (long)doubleValue();
			else
				throw new ClassCastException("Object reference could not be casted");
		}
		
		return 0;
	}

	/**
	 * Converts the stored value to a <code>float</code> representation and returns
	 * the new value. 
	 * 
	 * @return The converted <code>float</code> value. If the stored value is equal to 
	 * "true", the return value will be <code>1</code>. If the value could not be 
	 * converted the return value will be <code>0</code>. Otherwise, the return value 
	 * should be a value between <code>Float.MIN_VALUE</code> and 
	 * <code>Float.MAX_VALUE</code>.
	 * @exception ClassCastException if the stored value could not be converted to a known type.
	 */
	public float toFloat()
	{
		if (ref != null)
		{
			if (ref instanceof String)
			{
				String val = (String)ref;
				if (val.equals("true"))
					return 1f;
				return Float.parseFloat(val);
			}
			else if (ref instanceof Boolean)
				return (booleanValue() ? (float)1 : 0);
			else if (ref instanceof Byte) 
				return byteValue();
			else if (ref instanceof Character) 
				return charValue();
			else if (ref instanceof Short)
				return shortValue();
			else if (ref instanceof Integer) 
				return intValue();
			else if (ref instanceof Long) 
				return longValue();
			else if (ref instanceof Float) 
				return floatValue();
			else if (ref instanceof Double)
				return (float)doubleValue();
			else
				throw new ClassCastException("Object reference could not be casted");
		}
		
		return 0;
	}

	/**
	 * Converts the stored value to a <code>double</code> representation and returns
	 * the new value. 
	 * 
	 * @return The converted <code>double</code> value. If the stored value is equal to 
	 * "true", the return value will be <code>1</code>. If the value could not be 
	 * converted the return value will be <code>0</code>. Otherwise, the return value 
	 * should be a value between <code>Double.MIN_VALUE</code> and 
	 * <code>Double.MAX_VALUE</code>.
	 * @exception ClassCastException if the stored value could not be converted to a known type.
	 */
	public double toDouble()
	{
		if (ref != null)
		{
			if (ref instanceof String)
			{
				String val = (String)ref;
				if (val.equals("true"))
					return 1d;
				return Double.parseDouble(val);
			}
			else if (ref instanceof Boolean)
				return (booleanValue() ? (double)1 : 0);
			else if (ref instanceof Byte) 
				return byteValue();
			else if (ref instanceof Character) 
				return charValue();
			else if (ref instanceof Short)
				return shortValue();
			else if (ref instanceof Integer) 
				return intValue();
			else if (ref instanceof Long) 
				return longValue();
			else if (ref instanceof Float) 
				return floatValue();
			else if (ref instanceof Double)
				return doubleValue();
			else
				throw new ClassCastException("Object reference could not be casted");
		}
		
		return 0;
	}
	
}
