/*
 * @author VGonzales
 * Created on May 12, 2005
 * 
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.lang;

import java.util.*;

/**
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 * 
 * @version 1.0.0
 * @since AMXTools 1.0.0
 */
public class ObjectType
{
	/**
	 * 
	 */
	private final String name;

	/**
	 * Type safe enum constructor
	 * 
	 * @param name
	 */
	protected ObjectType(String name)
	{
		this.name = name;
	}

	/**
	 * Returns the name of the <code>ParamType</code> object.
	 * @return String
	 */	
	public String getName()
	{
		return ((this.name).toUpperCase());
	}
	
	/**
	 * Invalid type.
	 */
	public static final ObjectType INVALID = new ObjectType("INVALID");

	/**
	 * Boolean type.
	 */
	public static final ObjectType BOOL    = new ObjectType("BOOL");
	
	/**
	 * Byte type.
	 */
	public static final ObjectType BYTE    = new ObjectType("BYTE");
	
	/**
	 * Character type.
	 */
	public static final ObjectType CHAR    = new ObjectType("CHAR");
	
	/**
	 * Short type.
	 */
	public static final ObjectType SHORT   = new ObjectType("SHORT");
	
	/**
	 * Integer type.
	 */
	public static final ObjectType INTEGER = new ObjectType("INTEGER");
	
	/**
	 * Long type.
	 */
	public static final ObjectType LONG    = new ObjectType("LONG");
	
	/**
	 * Float type.
	 */
	public static final ObjectType FLOAT   = new ObjectType("FLOAT");
	
	/**
	 * Double type.
	 */
	public static final ObjectType DOUBLE  = new ObjectType("DOUBLE");
	
	/**
	 * String type.
	 */
	public static final ObjectType STRING  = new ObjectType("STRING");
	
	/**
	 * Object type.
	 */
	public static final ObjectType OBJECT  = new ObjectType("OBJECT");
	
	/**
	 * 
	 */
	private static final ObjectType[] PRIVATE_VALUES = 
	{
		BOOL, 
		BYTE, 
		CHAR, 
		SHORT, 
		INTEGER, 
		LONG,
		FLOAT,
		DOUBLE, 
		STRING,
		OBJECT 
	};
	 
	/**
	 * List of values belonging to this class type.
	 */
	public static final List VALUES = Collections.unmodifiableList(Arrays.asList(PRIVATE_VALUES));
		
	/*
	 *  (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		return getName();
	}	

	//Prevent subclasses from overriding these functions
	
	/*
	 *  (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public final boolean equals(Object obj)
	{
		return super.equals(obj);
	}
	
	/*
	 *  (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	public final int hashCode()
	{
		return super.hashCode();
	}
}
