/*
 * @author vgonzales
 * 
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.math;

/**
 * A utility class for scaling and adjusting level values.
 */
public class LevelUtil
{
	/**
	 * minimum level range value for modules.
	 */
	public static final int MIN_LVL = 0;
	
	/**
	 * maximum level range value for modules.
	 */
	public static final int MAX_LVL = 255;
	
	/**
	 * maximum decremental adjust value for modules.
	 */
	public static final int MIN_ADJUST = -255;
	
	/**
	 * maximum incremental adjust value for modules.
	 */
	public static final int MAX_ADJUST = 255;
	

	/**
	 * Step the level value by a calculated scale value or by one in either direction. Otherwise,
	 * set the level value to 50%.
	 * 
	 * @param nLevel
	 * @param nOffset
	 * @param deviceMin
	 * @param deviceMax
	 * @return
	 */
	public static int adjustLevel(int nLevel, int nOffset, int deviceMin, int deviceMax)
	{
//		diag("adjustLevel("+nLevel+","+nOffset+","+deviceMin+","+deviceMax+")");
		int nNewLevel = 0;
		
		// perform adjust
		int nStep = nOffset / MAX_LVL * (deviceMax - deviceMin + 1);

		// if the level is not set, assume it's 50%
		if (nLevel < 0)
		{
			nNewLevel = 128;
		}
		// perform device-scale step
		if (nOffset == 1 || nOffset == -1 || nStep == 0)
		{
			// Scale to device range
			int nTempLevel = RangeUtil.scaleRange(nLevel, MIN_LVL, MAX_LVL, deviceMin, deviceMax);
			
			// decrement level 1 step
			if (nOffset < 0 && nTempLevel > deviceMin)
				nTempLevel--;
			// increment level 1 step
			else if (nOffset > 0 && nTempLevel < deviceMax)
				nTempLevel++;
			
			// Scale back to SNAPI range
			nNewLevel = RangeUtil.scaleRange(nTempLevel, deviceMin, deviceMax, MIN_LVL, MAX_LVL);
		}
		// set new level value with the calculated step
		else
		{
			// Scale to device range
			int value = RangeUtil.scaleRange(nLevel, MIN_LVL, MAX_LVL, deviceMin, deviceMax);
			// Adjust the step
			nStep = RangeUtil.evaluateValue(value, nStep, deviceMin, deviceMax);
			// Scale back to SNAPI range
			nNewLevel = RangeUtil.scaleRange(nStep, deviceMin, deviceMax, MIN_LVL, MAX_LVL);
		}
		
		return nNewLevel;
	}

}
