package com.amx.duet.tools.net.http;
//#####################################################################//
//# Copyright Notice :                                                #//
//#    Copyright, AMX, Inc., 2008                                     #//
//#    Private, proprietary information, the sole property of AMX.    #//
//#    The contents, ideas, and concepts expressed herein are not to  #//
//#    be disclosed except within the confines of a confidential      #//
//#    relationship and only then on a need to know basis.            #//
//#####################################################################//
import java.net.HttpURLConnection;

/**
 * Public interface for any class wanting to use the HttpPostWrapper class.
 * @author Cnina
 *
 */
public interface IHttpPostWrapper
{
	/**
	 * This callback method will provide you with state information for the HTTP thread matching the 
	 * handle returned. The <code>state</code> parameters are defined in the <code>HTTPPostWrapper</code> class.
	 * The <code>HTTP_POST_SUCCESS</code> state parameter will always be returned after a successful post request/reply has occured
	 * and can be used as a message complete indicator.
	 * @param handle
	 * @param state
	 */
	public void handleHttpPostState(long handle, int state);
	/**
	 * This callback method will provide the returned/reply data from a successful post request matching the HTTP thread handle. 
	 * This event will trigger before any <code>HTTP_POST_SUCCESS</code> state event.
	 * <p> For simplicity, the <code>data</code> returned from the server is presented as a String. You may extract the same information from within the urlConn object by invoking the <code>getInputStream()</code> method.
	 * The urlConn object is returned in order to provide any header field information for the returned data (if needed).
	 * @param handle
	 * @param data
	 * @param urlConn
	 */
	public void handleHttpPostReply(long handle, String data, HttpURLConnection urlConn);
	
}
