/*
 * @author VGonzales
 * 
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.net.snmp;

import java.io.IOException;
import java.io.PipedReader;
import java.io.PipedWriter;
import java.io.PrintWriter;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.UnknownHostException;

import com.amx.duet.tools.lang.Logger;
import com.amx.duet.tools.net.snmp.api.SNMPBadValueException;
import com.amx.duet.tools.net.snmp.api.SNMPObjectIdentifier;
import com.amx.duet.tools.net.snmp.api.SNMPTrapReceiverInterface;
import com.amx.duet.tools.net.snmp.api.SNMPv1TrapListener;
import com.amx.duet.tools.net.snmp.api.SNMPv1TrapPDU;
import com.amx.duet.tools.net.snmp.api.SNMPv2TrapListener;
import com.amx.duet.tools.net.snmp.api.SNMPv2TrapPDU;

/**
 * This class is not complete...
 */
public class SNMPTrapConnection extends Thread implements SNMPv1TrapListener, SNMPv2TrapListener
{
	private static final int MAX_VERSION = 2;
	
    SNMPTrapReceiverInterface trapReceiverInterface = null;
    SNMPTrapConnectionListener m_listener = null;
    
    PipedReader errorReader;
    PipedWriter errorWriter;
    
//	private InetAddress m_hostAddress = null;
//	private String m_community = "";
	private int m_version = 1;
	private SNMPObjectIdentifier m_DeviceMakeID = null;
	
    private int nDebug = Logger.LOG_ERROR;
    
    /**
     * 
     * @param version
     * @param hostAddress
     * @param deviceMakeOID
     * @param community
     * @param listener
     */
	public SNMPTrapConnection(int version, InetAddress hostAddress, String deviceMakeOID, String community, SNMPTrapConnectionListener listener)
    {
		initConnection(version, hostAddress, deviceMakeOID, community, listener);
    }
    
	/**
	 * 
	 * @param version
	 * @param hostAddress
	 * @param deviceMakeOID
	 * @param community
	 * @param listener
	 */
	public SNMPTrapConnection(int version, String hostAddress, String deviceMakeOID, String community, SNMPTrapConnectionListener listener)
	{
		initConnection(version, hostAddress, deviceMakeOID, community, listener);
	}
	
	/**
	 * 
	 * @param hostAddress
	 * @param deviceMakeOID
	 * @param community
	 * @param listener
	 */
	public SNMPTrapConnection(InetAddress hostAddress, String deviceMakeOID, String community, SNMPTrapConnectionListener listener)
	{
		initConnection(m_version, hostAddress, deviceMakeOID, community, listener);
	}
	
	/**
	 * 
	 * @param hostAddress
	 * @param deviceMakeOID
	 * @param community
	 * @param listener
	 */
	public SNMPTrapConnection(String hostAddress, String deviceMakeOID, String community, SNMPTrapConnectionListener listener)
	{
		initConnection(m_version, hostAddress, deviceMakeOID, community, listener);
	}
	
	/*
	 * 
	 * @param version
	 * @param hostAddress
	 * @param community
	 * @param listener
	 */
	private void initConnection(int version, String hostAddress, String deviceMakeOID, String community, SNMPTrapConnectionListener listener)
	{
		try
		{
			initConnection(version, InetAddress.getByName(hostAddress), deviceMakeOID, community, listener);
		}
		catch (UnknownHostException e)
		{
			error("UnknownHostException - " + e.toString());
		}
	}
	
	/*
	 * 
	 * @param version
	 * @param hostAddress
	 * @param community
	 * @param listener
	 * @throws IllegalArgumentException
	 */
	private void initConnection(int version, InetAddress hostAddress, String deviceMakeOID, String community, SNMPTrapConnectionListener listener)
	{
		if (listener == null)
		{
			error("Invalid listener");
			return;
		}
		
		if (hostAddress == null)
		{
			error("Invalid host address");
			return;
		}
		
		if (community == null || community.length() == 0)
		{
			error("Invalid community");
			return;
		}
		
		if (version <= 0 && version > MAX_VERSION)
		{
			error("Invalid version number; must be 1 or 2");
			return;
		}
		
		if (!setDeviceMakeOID(deviceMakeOID))
		{
			error("Invalid device make OID");
			return;
		}
		
		m_version = version - 1;
//		m_hostAddress = hostAddress;
//		m_community = community;
		m_listener = listener;
		
        try
		{
            errorReader = new PipedReader();
			errorWriter = new PipedWriter(errorReader);
			this.start();
            
        	trapReceiverInterface = new SNMPTrapReceiverInterface(new PrintWriter(errorWriter));
            trapReceiverInterface.addv1TrapListener(this);
            trapReceiverInterface.addv2TrapListener(this);
		}
		catch (SocketException e)
		{
			error("Could not establish trap connection - " + e.toString());
		}
		catch (IOException e)
		{
			error("Trap connection could not be started - " + e.toString());
		}
	}

	/**
	 * Defines the device's company Object Identifier. The company OID should be 
	 * the prefix used for every OID used in the device's list of managed 
	 * objects in its MIB hierarchy. 
	 * <p>
	 * The MIB hierarchy can be depicted as a tree with a nameless root, the 
	 * levels of which are assigned by different organizations. Vendors usually 
	 * define private branches that include managed objects for their products. 
	 * The level prior to the products branch is the company's OID.
	 * <p>
	 * Example:
	 * <pre>
	 * If the OID to get to a device variable looked like this:
	 * 
	 * 1.3.6.1.4.1.9.3.3.1
	 * 
	 * The product OID might be:
	 * 
	 * 1.3.6.1.4.1.9.3
	 * 
	 * And the company OID might be:
	 * 
	 * 1.3.6.1.4
	 * 
	 * The call should look like this:
	 * boolean success = setDeviceMakeOID("1.3.6.1.4");
	 * </pre>
	 * Basically, the company OID is the prefix ID for the product ID and the 
	 * product ID is the prefix for the variable OID.
	 * 
	 * @param makeOID String containing the company OID.
	 * @return <code>true</code> if the OID was valid; <code>false</code> 
	 * otherwise.
	 */
	public boolean setDeviceMakeOID(String makeOID)
	{
		boolean bSuccess = false;
		try
		{
			m_DeviceMakeID = new SNMPObjectIdentifier(makeOID);
			if (m_DeviceMakeID != null)
				bSuccess = true;
		}
		catch (SNMPBadValueException e)
		{
			error("Invalid OID for setDeviceMakeOID(): " + e.toString());
		}
		return bSuccess;
	}

	/**
	 * Retrieves the device's company Object Identifier in a String object.
	 * 
	 * @return String containing the company OID.
	 */
	public String getDeviceMakeOID()
	{
		String oid = "";
		if (m_DeviceMakeID != null)
			oid = m_DeviceMakeID.toString();
		return oid;
	}
	
	/**
     * 
     *
     */
    public void startReceiving()
    {
    	if (trapReceiverInterface != null)
    		trapReceiverInterface.startReceiving();
    }
	
    /**
     * 
     *
     */
    public void stopReceiving()
    {
		try
		{
	    	if (trapReceiverInterface != null)
	    		trapReceiverInterface.stopReceiving();
		}
		catch (Exception e)
		{
			error("Problem occurred while stopping trap receiver - " + e.toString());
		}
    }
	
	/* (non-Javadoc)
	 * @see com.amx.duet.tools.net.snmp.SNMPv1TrapListener#processv1Trap(com.amx.duet.tools.net.snmp.SNMPv1TrapPDU)
	 */
	public void processv1Trap(SNMPv1TrapPDU trapPDU)
	{
		debug("processv1Trap(" + trapPDU + ")");
		if (m_listener != null)
			m_listener.handleSNMPTrap(trapPDU.getEnterpriseOID().toString(), null);
	}

	/* (non-Javadoc)
	 * @see com.amx.duet.tools.net.snmp.SNMPv2TrapListener#processv2Trap(com.amx.duet.tools.net.snmp.SNMPv2TrapPDU)
	 */
	public void processv2Trap(SNMPv2TrapPDU trapPDU)
	{
		debug("processv2Trap(" + trapPDU + ")");
		if (m_listener != null)
			m_listener.handleSNMPTrap(trapPDU.getSNMPTrapOID().toString(), null);
	}

//	private SNMPObjectIdentifier createOID(String itemID)
//	{
//		SNMPObjectIdentifier oid = null;
//		String make = getDeviceMakeOID();
//		try
//		{
//			if (make.length() > 0)
//				oid = new SNMPObjectIdentifier(make + "." + itemID);
//			else
//				oid = new SNMPObjectIdentifier(itemID);
//		}
//		catch (SNMPBadValueException e)
//		{
//			error("Invalid OID - " + e.toString());
//		}
//		return oid;
//	}

//	/*
//	 * 
//	 * @param names
//	 * @return
//	 */
//	private SNMPObjectIdentifier[] createOID(String[] itemIDs)
//	{
//		SNMPObjectIdentifier[] oids = null;
//		if (itemIDs.length > 0)
//		{
//			oids = new SNMPObjectIdentifier[itemIDs.length];
//			for(int i = 0; i < itemIDs.length; i++)
//				oids[i] = createOID(itemIDs[i]);
//			
//		} // if (names.length > 0)
//		
//		return oids;
//	}

	/**
	 * 
	 *
	 */
	public void dispose()
	{
		if (trapReceiverInterface != null)
		{
			try
			{
				trapReceiverInterface.removev1TrapListener(this);
				trapReceiverInterface.removev2TrapListener(this);
				trapReceiverInterface.dispose();
				trapReceiverInterface = null;
			}
			catch (Exception e)
			{
				error("Stopped trap listener - " + e.toString());
			}
		}
		
		m_listener = null;
		this.interrupt();
	}

	/* (non-Javadoc)
	 * @see java.lang.Thread#run()
	 */
	public void run()
	{
		debug("run()");
        int numChars;
        char[] charArray = new char[256];
        
        try
        {
            while (!this.isInterrupted() && ((numChars = errorReader.read(charArray, 0, charArray.length)) != -1))
            {
                debug("Problem receiving trap or inform:\n");
                debug(new String(charArray, 0, numChars));
            }
        }
        catch(IOException e)
        {
            error("Problem receiving errors; error reporter exiting!");
        }
	}

//	private void diag(String str)
//	{
//		System.out.println("SNMPTrapConnection: " + str);
//	}

	private void debug(String str)
	{
		if (nDebug > Logger.LOG_WARNING)
			System.out.println("SNMPTrapConnection: " + str);
	}
	
	private void error(String str)
	{
		System.out.println("ERROR - SNMPTrapConnection: " + str);
	}
	
	/**
	 * @deprecated Do not use this method; use setDebugState instead
	 * @param bState
	 */
	public void setDebug(boolean bState)
	{
		if (bState)
			nDebug = Logger.LOG_DEBUG;
		else
			nDebug = Logger.LOG_ERROR;
	}
	
	/**
	 * Puts the class into a debug diagnostic state.
	 * 
	 * @param nState
	 */
	public void setDebugState(int nState)
	{
		nDebug = nState;
	}
	
	/**
	 * Gets the current diagnostic state.
	 * 
	 * @return
	 */
	public int getDebugState()
	{
		return nDebug;
	}
	
}
