/*
 * @author vgonzales
 * 
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.text;

/**
 * A utility class for converting String hex values to integers and integer to 
 * String hex values.
 */
public class HexUtil
{
    /**
     * Converts an ASCII string containing the hexadecimal representation of a number to an integer. 
     * 
     * @param szValue a string in the form of "ffff" (not case sensitive)
     * @return An integer representing the converted string. Any non-hexadecimal characters in the 
     * string are ignored. hextoi returns a value representing the first complete set of characters that 
     * represents an integer. Valid characters are "0" through "9", "A" through "F" and "a" through "f". 
     * If no valid characters are found, zero is returned as a result.
     */
	public final static int hextoi(String hex)
	{
	    char HexMap[] = new char[] { '0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
	    String szStr = hex.toUpperCase();
	    int MAX_CHARS = HexMap.length;
	    int nResult = Integer.MIN_VALUE;
	    
	    int start = szStr.indexOf("0X");
		if (start >= 0)
		{
			szStr = szStr.substring(2);
		}
		
	    if (szStr == null || szStr.length() < 1)
	    	return Integer.MIN_VALUE;
	    
    	boolean bFound = false;
	    for (int i = 0; i < szStr.length(); i++)
	    {
	    	bFound = false; // reset this for every character
	        for (int j = 0; j < MAX_CHARS; j++)
	        {
	            if (szStr.charAt(i) == HexMap[j])
	            {
                    nResult <<= 4;
	                nResult |= j;
	                bFound = true;
	                break;
	            }
	        }
	        
	        // Not a valid base 16 character
	        if (!bFound)
	        {
		    	return Integer.MIN_VALUE;
	        }
	    }
	    
	    return nResult;
	}
	
	public final static int hextoi(String hex, boolean big)
	{
	    char HexMap[] = new char[] { '0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
	    String szStr = hex.toUpperCase();
	    int MAX_CHARS = HexMap.length;
	    int nResult = Integer.MIN_VALUE;
	    
	    int start = szStr.indexOf("0X");
		if (start >= 0)
		{
			szStr = szStr.substring(2);
		}
		
	    if (szStr == null || szStr.length() < 1)
	    	return Integer.MIN_VALUE;
	    
	    // If little endian flip every other character.
	    if (!big)
	    {
	    	if ((szStr.length() % 2) != 0)
	    		szStr = "0" + szStr;
	    	String temp = "";
	    	for (int x = szStr.length()-1; x >= 0; x=x-2)
	    	{
	    		temp += szStr.charAt(x-1);
	    		temp += szStr.charAt(x);
	    	}
	    	szStr = temp;
	    }
	    
    	boolean bFound = false;
	    for (int i = 0; i < szStr.length(); i++)
	    {
	    	bFound = false; // reset this for every character
	        for (int j = 0; j < MAX_CHARS; j++)
	        {
	            if (szStr.charAt(i) == HexMap[j])
	            {
                    nResult <<= 4;
	                nResult |= j;
	                bFound = true;
	                break;
	            }
	        }
	        
	        // Not a valid base 16 character
	        if (!bFound)
	        {
		    	return Integer.MIN_VALUE;
	        }
	    }
	    
	    return nResult;
	}
	
	/**
	 * Converts an integer to an ASCII string containing the hexadecimal representation of the number.
	 * 
	 * @param value integer to be converted to a hexadecimal string.
	 * @return A character string that contains the hexadecimal representation of the specified integer.
	 */
	public final static String itohex(int value)
	{
		String str = "";
		
		try
		{
			str = Integer.toHexString(value);
		}
		catch (Exception e) {} // Ignore the exception
		
		return str;
	}
	
	/**
	 * Changes two ascii bytes into one integer value.
	 * Ex: 0x37,0x43 -> 0x7C = 124
	 * 
	 * @param cZa [0x30..0x39,0x41..0x46]
	 * @param cZb [0x30..0x39,0x41..0x46]
	 * @return integer
	 */
	public static int HEXTOI(byte cZa, byte cZb)
	{
		int a, b;
		
		if ( (cZa < 0x30) || (cZa > 0x46) || ( cZa > 0x39 && cZa < 0x41 ) )
		{
			error("First parameter was invalid (number range should be 048-057, 065-070. You entered: " + cZa);
			return Integer.MIN_VALUE;
		}	
		if ( (cZb < 0x30) || (cZb > 0x46) || ( cZb > 0x39 && cZb < 0x41 ) )
		{
			error("Second parameter was invalid (number range should be 048-057, 065-070. You entered: " + cZb);
			return Integer.MIN_VALUE;
		}

		if (cZa < 0x40) { a = cZa - 0x30; }            //VALUE IS 0-9
		else { a = cZa - 0x41 + 10; }     					//VALUE IS A-F (even though A-F start at 0x41, 10 is artifically added by starting at 0x37)
		if (cZb < 0x40) { b = cZb - 0x30; }            //VALUE IS 0-9
		else { b = cZb - 0x41 + 10; }                       //VALUE IS A-F

		return ((a << 4) + b);
	}
	
	/**
	 * Changes two ascii chars into one integer value.
	 * Ex: 0x37,0x43 -> 0x7C = 124
	 * 
	 * @param cZa ['0'..'9','A'..'F']
	 * @param cZb ['0'..'9','A'..'F']
	 * @return integer
	 */
	public static int HEXTOI(char cZa, char cZb)
	{
		int a, b;
		cZa = Character.toUpperCase(cZa);
		cZb = Character.toUpperCase(cZb);
		
		if ( ((byte)cZa < 0x30) || ((byte)cZa > 0x46) || ( ((byte)cZa > 0x39) && ((byte)cZa < 0x41 )) )
		{
			error("First parameter was invalid (number range should be '1'-'9', 'A'-'F'. You entered: " + cZa);
			return Integer.MIN_VALUE;
		}
		if ( ((byte)cZb < 0x30) || ((byte)cZb > 0x46) || ( ((byte)cZb > 0x39) && ((byte)cZb < 0x41 )) )
		{
			error("First parameter was invalid (number range should be '1'-'9', 'A'-'F'. You entered: " + cZb);
			return Integer.MIN_VALUE;
		}

		if ((byte)cZa < 0x40) { a = (byte)cZa - 0x30; }            //VALUE IS 0-9
		else { a = (byte)cZa - 0x41 + 10; }     					//VALUE IS A-F (even though A-F start at 0x41, 10 is artifically added by starting at 0x37)
		if ((byte)cZb < 0x40) { b = (byte)cZb - 0x30; }            //VALUE IS 0-9
		else { b = (byte)cZb - 0x41 + 10; }                       //VALUE IS A-F

		return ((a << 4) + b);
	}

	/**
	 * Changes two ascii bytes into one ascii character.
	 * Ex: 0x34,0x31 -> 0x41 = 'A'
	 * 
	 * @param cZa
	 * @param cZb
	 * @return char
	 */
	public static char HEXTOA(byte cZa, byte cZb)
	{	
		int a, b;
		
		if ( (cZa < 0x30) || (cZa > 0x46) || ( cZa > 0x39 && cZa < 0x41 ) )
		{
			error("First parameter was invalid (number range should be 048-057, 065-070. You entered: " + cZa);
			return Character.MIN_VALUE;
		}	
		if ( (cZb < 0x30) || (cZb > 0x46) || ( cZb > 0x39 && cZb < 0x41 ) )
		{
			error("Second parameter was invalid (number range should be 048-057, 065-070. You entered: " + cZb);
			return Character.MIN_VALUE;
		}

		if (cZa < 0x40) { a = cZa - 0x30; }            //VALUE IS 0-9
		else { a = cZa - 0x37; }                       //VALUE IS A-F
		if (cZb < 0x40) { b = cZb - 0x30; }            //VALUE IS 0-9
		else { b = cZb - 0x37; }                       //VALUE IS A-F

		return (char)((a << 4) + b);
	}

	/*
	 * 
	 * @param str
	 */
	private static void error(String str)
	{
		System.out.println("HexUtil: ERROR - " + str);
	}
}
