/*
 * @author VGonzales
 * Created on Apr 25, 2005
 * 
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.util;

import com.amx.duet.core.master.netlinx.DPS;
import com.amx.duet.core.master.netlinx.Event;
import com.amx.duet.da.NetLinxDevice;
import com.amx.duet.tools.lang.Logger;

/**
 * A utility class, which contains a few Duet specific methods. Class containing 
 * miscellaneous static methods that can be used in Duet Modules.
 * 
 * @version 1.0.3
 * @since AMXTools 1.0.0
 *
 */
public class DuetUtil
{
	/**
	 * Converts the event data value (if it is a string) to a <code>String</code> object
	 * 
	 * @param caller Class object that called this method
	 * @param evt Duet <code>Event</code> object containing the data
	 * @return String converted from a byte or character array
	 */
	public final static String eventValToStr(Object caller, Event evt)
	{
		String str = "";
		try
		{
			if (evt.dataType == Event.D_STRING8)
			{
				str = new String((byte[])evt.dataValue);
			}
			else if (evt.dataType == Event.D_STRING16)
			{
				str = new String((char[])evt.dataValue);
			}
		}
		catch (Exception e)
		{
			Logger.error(caller, "Exception thrown! " + e.getMessage());
		}
		
		return str;
	}

	/**
	 * Creates a new virtual device based on the DPS string passed in.
	 * 
	 * @param vdStr String containing a DPS value (ie "0:1:0")
	 * @return a <code>NetLinxDevice</code> with the vitual flag set to true
	 */
	public final static NetLinxDevice createVirtualDevice(String vdStr)
	{
		DPS dps = new DPS(vdStr);
		return new NetLinxDevice(dps, true);
	}

	/**
	 * Creates a new loop back device based on the DPS string passed in.  The device can
	 * be either virtual or physcial based on the virtual flag.
	 * 
	 * @param vdStr String containing a DPS value (ie "0:1:0")
	 * @param bVirtual boolean value used to register the device as virtual or physical
	 * @return a newly created <code>LoopBackDevice</code>
	 */
	public final static LoopBackDevice createLoopBackDevice(String vdStr, boolean bVirtual)
	{
		DPS dps = new DPS(vdStr);
		return new LoopBackDevice(dps, bVirtual);
	}
	
}
