/*
 * @author VGonzales
 * 
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.util;

import java.util.*;

import com.amx.duet.core.master.netlinx.DPS;
import com.amx.duet.core.master.netlinx.Event;
import com.amx.duet.core.master.netlinx.IButtonListener;
import com.amx.duet.core.master.netlinx.IChannelListener;
import com.amx.duet.core.master.netlinx.ICustomListener;
import com.amx.duet.core.master.netlinx.IDataListener;
import com.amx.duet.core.master.netlinx.ILevelListener;
import com.amx.duet.core.master.netlinx.Level;
import com.amx.duet.da.NetLinxDevice;

/**
 * The <code>LoopBackDevice</code> class resembles every aspect of the <code>NetLinxDevice</code> class
 * except that it provides event data back to any object that is listening for the device events triggered 
 * by this device.  Much like in NetLinx, a Duet object can use this class to recieve the data that it is
 * sending out.  Also, any object that instantiates a <code>LoopBackDevice</code> with the same DPS as 
 * another <code>NetLinxDevice</code> can also receive the other device's events.
 * 
 * @version 1.0.0
 * @since AMXTools 1.0.1
 *
 */
public class LoopBackDevice extends NetLinxDevice
{
	private Vector m_channelListeners = new Vector(1);
	private Vector m_buttonListeners = new Vector(1);
	private Vector m_levelListeners = new Vector(1);
	private Vector m_dataListeners = new Vector(1);
	private Vector m_customListeners = new Vector(1);

    /**
     * Creates a loop back <code>NetLinxDevice</code> given a DPS object and a flag  
     * indicating whether or not this is a virtual device. 
     * 
     * @param dps
     * @param virtualDevice
     */
    public LoopBackDevice(DPS dps, boolean virtualDevice)
    {
        super(dps, virtualDevice);
    }

    /**
     * Creates a loop back <code>NetLinxDevice</code>.
     */
    public LoopBackDevice()
    {
        super();
    }

    /* (non-Javadoc)
     * @see com.amx.duet.core.master.IcspNetLinxDevice#addButtonListener(com.amx.duet.core.master.netlinx.IButtonListener)
     */
    public void addButtonListener(IButtonListener buttonListener)
    {
        super.addButtonListener(buttonListener);
		this.m_buttonListeners.add(buttonListener);
    }

    /* (non-Javadoc)
     * @see com.amx.duet.core.master.IcspNetLinxDevice#addChannelListener(com.amx.duet.core.master.netlinx.IChannelListener)
     */
    public void addChannelListener(IChannelListener channelListener)
    {
        super.addChannelListener(channelListener);
		this.m_channelListeners.add(channelListener);
    }

    /* (non-Javadoc)
     * @see com.amx.duet.core.master.IcspNetLinxDevice#addCustomListener(com.amx.duet.core.master.netlinx.ICustomListener)
     */
    public void addCustomListener(ICustomListener customListener)
    {
        super.addCustomListener(customListener);
		this.m_customListeners.add(customListener);
    }

    /* (non-Javadoc)
     * @see com.amx.duet.core.master.IcspNetLinxDevice#addDataListener(com.amx.duet.core.master.netlinx.IDataListener)
     */
    public void addDataListener(IDataListener dataListener)
    {
        super.addDataListener(dataListener);
		this.m_dataListeners.add(dataListener);
    }

    /* (non-Javadoc)
     * @see com.amx.duet.core.master.IcspNetLinxDevice#addLevelListener(com.amx.duet.core.master.netlinx.ILevelListener)
     */
    public void addLevelListener(ILevelListener levelListener)
    {
        super.addLevelListener(levelListener);
		this.m_levelListeners.add(levelListener);
    }

//	/*
//	 * This method will allow duet modules to receive button events fired from a <code>NetLinxDevice</code>. 
//	 * This makes it possible for a module to listen to its own events.
//	 * 
//	 * @param eObj <code>Event</code> object to be received
//	 */
//    private void loopbackButtonEvent(Event eObj)
//    {
//		Enumeration e = m_buttonListeners.elements();
//		while(e.hasMoreElements())
//		{
//			try
//			{
//				((IButtonListener)(e.nextElement())).handleButtonEvent(eObj);
//			}
//			catch (NoSuchElementException ex)
//			{
//				throw ex;
//			}
//		}
//    }

	/*
	 * This method will allow duet modules to receive channel events fired from a <code>NetLinxDevice</code>. 
	 * This makes it possible for a module to listen to its own events.
	 * 
	 * @param eObj <code>Event</code> object to be received
	 */
    private void loopbackChannelEvent(Event eObj)
    {
		Enumeration e = m_channelListeners.elements();
		while(e.hasMoreElements())
		{
			try
			{
				((IChannelListener)(e.nextElement())).handleChannelEvent(eObj);
			}
			catch (NoSuchElementException ex)
			{
				throw ex;
			}
		}
    }

	/*
	 * This method will allow duet modules to receive custom events fired from a <code>NetLinxDevice</code>. 
	 * This makes it possible for a module to listen to its own events.
	 * 
	 * @param eObj <code>Event</code> object to be received
	 */
    private void loopbackCustomEvent(Event eObj)
    {
		Enumeration e = m_customListeners.elements();
		while(e.hasMoreElements())
		{
			try
			{
				((ICustomListener)(e.nextElement())).handleCustomEvent(eObj);
			}
			catch (NoSuchElementException ex)
			{
				throw ex;
			}
		}
    }

	/*
	 * This method will allow duet modules to receive data events fired from a <code>NetLinxDevice</code>. 
	 * This makes it possible for a module to listen to its own events.
	 * 
	 * @param eObj <code>Event</code> object to be received
	 */
    private void loopbackDataEvent(Event eObj)
    {
		Enumeration e = m_dataListeners.elements();
		while(e.hasMoreElements())
		{
			try
			{
				((IDataListener)(e.nextElement())).handleDataEvent(eObj);
			}
			catch (NoSuchElementException ex)
			{
				throw ex;
			}
		}
    }

	/*
	 * This method will allow duet modules to receive level events fired from a <code>NetLinxDevice</code>. 
	 * This makes it possible for a module to listen to its own events.
	 * 
	 * @param eObj <code>Event</code> object to be received
	 */
    private void loopbackLevelEvent(Event eObj)
    {
        super.handleLevelEvent(eObj);

		Enumeration e = m_levelListeners.elements();
		while(e.hasMoreElements())
		{
			try
			{
				((ILevelListener)(e.nextElement())).handleLevelEvent(eObj);
			}
			catch (NoSuchElementException ex)
			{
				throw ex;
			}
		}
    }

    /* (non-Javadoc)
     * @see com.amx.duet.core.master.IcspNetLinxDevice#notifyOffline()
     */
    public void notifyOffline()
    {
        super.notifyOffline();
        
        loopbackDataEvent(EventPacker.packOffline(getDPS()));
    }

    /* (non-Javadoc)
     * @see com.amx.duet.core.master.IcspNetLinxDevice#notifyOnline()
     */
    public void notifyOnline()
    {
        super.notifyOnline();

        loopbackDataEvent(EventPacker.packOnline(getDPS()));
    }

    /* (non-Javadoc)
     * @see com.amx.duet.core.master.IcspNetLinxDevice#off(int)
     */
    public void off(int channelId)
    {
        super.off(channelId);

        loopbackChannelEvent(EventPacker.packOff(getDPS(), channelId));
    }

    /* (non-Javadoc)
     * @see com.amx.duet.core.master.IcspNetLinxDevice#offInputChannel(int)
     */
    public void offInputChannel(int channelId)
    {
        super.offInputChannel(channelId);

        loopbackChannelEvent(EventPacker.packOffInputChannel(getDPS(), channelId));
    }

    /* (non-Javadoc)
     * @see com.amx.duet.core.master.IcspNetLinxDevice#on(int)
     */
    public void on(int channelId)
    {
        super.on(channelId);

        loopbackChannelEvent(EventPacker.packOn(getDPS(), channelId));
    }

    /* (non-Javadoc)
     * @see com.amx.duet.core.master.IcspNetLinxDevice#onInputChannel(int)
     */
    public void onInputChannel(int channelId)
    {
        super.onInputChannel(channelId);

        loopbackChannelEvent(EventPacker.packOnInputChannel(getDPS(), channelId));
    }

    /* (non-Javadoc)
     * @see com.amx.duet.core.master.IcspNetLinxDevice#sendCommand(byte[])
     */
    public void sendCommand(byte[] str)
    {
        super.sendCommand(str);

        loopbackDataEvent(EventPacker.packSendCommand(getDPS(), str));
    }

    /* (non-Javadoc)
     * @see com.amx.duet.core.master.IcspNetLinxDevice#sendCommand(char[])
     */
    public void sendCommand(char[] str)
    {
        super.sendCommand(str);

        loopbackDataEvent(EventPacker.packSendCommand(getDPS(), str));
    }

    /* (non-Javadoc)
     * @see com.amx.duet.core.master.IcspNetLinxDevice#sendCommand(java.lang.String)
     */
    public void sendCommand(String str)
    {
        super.sendCommand(str);

        loopbackDataEvent(EventPacker.packSendCommand(getDPS(), str));
    }

    /* (non-Javadoc)
     * @see com.amx.duet.core.master.IcspNetLinxDevice#sendCommandWide(java.lang.String)
     */
    public void sendCommandWide(String str)
    {
        super.sendCommandWide(str);

        loopbackDataEvent(EventPacker.packSendCommandWide(getDPS(), str));
    }

    /* (non-Javadoc)
     * @see com.amx.duet.core.master.IcspNetLinxDevice#sendCustom(int, int, int, int, int, int, byte[], byte[])
     */
    public void sendCustom(int id, int eventType, int flag, int value1, int value2, int value3, byte[] eventData, byte[] encode)
    {
        super.sendCustom(
            id,
            eventType,
            flag,
            value1,
            value2,
            value3,
            eventData,
            encode);

        loopbackCustomEvent(EventPacker.packSendCustom(
        	getDPS(), 
        	id, 
        	eventType, 
        	flag, 
        	value1, 
        	value2, 
        	value3, 
        	eventData, 
        	encode));
    }

    /* (non-Javadoc)
     * @see com.amx.duet.core.master.IcspNetLinxDevice#sendCustom(int, int, int, int, int, int, char[], byte[])
     */
    public void sendCustom(int id, int eventType, int flag, int value1, int value2, int value3, char[] eventData, byte[] encode)
    {
        super.sendCustom(
            id,
            eventType,
            flag,
            value1,
            value2,
            value3,
            eventData,
            encode);

        loopbackCustomEvent(EventPacker.packSendCustom(
        	getDPS(), 
        	id, 
        	eventType, 
        	flag, 
        	value1, 
        	value2, 
        	value3, 
        	eventData, 
        	encode));
    }

    /* (non-Javadoc)
     * @see com.amx.duet.core.master.IcspNetLinxDevice#sendCustom(int, int, int, int, int, int, java.lang.String, byte[])
     */
    public void sendCustom(int id, int eventType, int flag, int value1, int value2, int value3, String eventData, byte[] encode)
    {
        super.sendCustom(
            id,
            eventType,
            flag,
            value1,
            value2,
            value3,
            eventData,
            encode);

        loopbackCustomEvent(EventPacker.packSendCustom(
        	getDPS(), 
        	id, 
        	eventType, 
        	flag, 
        	value1, 
        	value2, 
        	value3, 
        	eventData, 
        	encode));
    }

    /* (non-Javadoc)
     * @see com.amx.duet.core.master.IcspNetLinxDevice#sendCustomWide(int, int, int, int, int, int, java.lang.String, byte[])
     */
    public void sendCustomWide(int id, int eventType, int flag, int value1, int value2, int value3, String eventData, byte[] encode)
    {
        super.sendCustomWide(
            id,
            eventType,
            flag,
            value1,
            value2,
            value3,
            eventData,
            encode);

        loopbackCustomEvent(EventPacker.packSendCustomWide(
        	getDPS(), 
        	id, 
        	eventType, 
        	flag, 
        	value1, 
        	value2, 
        	value3, 
        	eventData, 
        	encode));
    }

    /* (non-Javadoc)
     * @see com.amx.duet.core.master.IcspNetLinxDevice#sendLevel(int, com.amx.duet.core.master.netlinx.Level)
     */
    public void sendLevel(int levelId, Level l)
    {
        super.sendLevel(levelId, l);

        loopbackLevelEvent(EventPacker.packSendLevel(getDPS(), levelId, l));
    }

    /* (non-Javadoc)
     * @see com.amx.duet.core.master.IcspNetLinxDevice#sendString(byte[])
     */
    public void sendString(byte[] str)
    {
        super.sendString(str);

        loopbackDataEvent(EventPacker.packSendString(getDPS(), str));
    }

    /* (non-Javadoc)
     * @see com.amx.duet.core.master.IcspNetLinxDevice#sendString(char[])
     */
    public void sendString(char[] str)
    {
        super.sendString(str);

        loopbackDataEvent(EventPacker.packSendString(getDPS(), str));
    }

    /* (non-Javadoc)
     * @see com.amx.duet.core.master.IcspNetLinxDevice#sendString(java.lang.String)
     */
    public void sendString(String str)
    {
        super.sendString(str);

        loopbackDataEvent(EventPacker.packSendString(getDPS(), str));
    }

    /* (non-Javadoc)
     * @see com.amx.duet.core.master.IcspNetLinxDevice#sendStringWide(java.lang.String)
     */
    public void sendStringWide(String str)
    {
        super.sendStringWide(str);

        loopbackDataEvent(EventPacker.packSendStringWide(getDPS(), str));
    }

//	private void diag(String str)
//	{
//		Diag.diag(this, str);
//	}
}
