/*
 * @author VGonzales
 * Created on May 24, 2005
 * 
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.util;

import java.util.LinkedList;

import com.amx.duet.da.NetLinxDevice;
import com.amx.duet.util.Timer;
import com.amx.duet.util.TimerListener;

/**
 * This class was created to be a <code>NetLinxDevice</code> pulse manager. It allows the user
 * to set the delay between channel on and off commands. It also queues up each pulse command 
 * sent to it, so that no commands are dropped.
 * 
 * @version 1.0.0
 * @since AMXTools 1.0.0
 * 
 */
public class Pulser //extends DuetUtil
{
	// Place each pulse request onto this queue
	LinkedList queue = new LinkedList();
	
	// The default PULSE time is 5 (0.5 seconds)
	protected int nPulseTime = 5;
	
	private class devpulse implements TimerListener
	{
		public NetLinxDevice device = null;
		public int channel = 0;

		// Timers will be created and deleted as each pulse is sent
		private Timer timerPulse = null;
		
		/*
		 * The constructor will setup the members and then create a new
		 * timer, which will be destroy once the event is fired off
		 */
		public devpulse(NetLinxDevice dev, int nChannel)
		{
			device = dev;
			channel = nChannel;
			
			if (device != null)
			{
				// Turn channel on
				device.on(channel);
				timerPulse = new Timer(this, nPulseTime);
				
				// Start the timer event immediately
				if (timerPulse != null)
					timerPulse.start();
			}
		}

		public void handleTimerEvent(Timer timer)
		{
			if (timerPulse == timer)
			{
				if (device != null)
				{
					// Turn channel off
					device.off(channel);
					
					// Call the remove callback method to take this 
					// object off of the queue
					remove();
					
					// Clean up timer memory
					timerPulse = null;
				}
			}
		}
	}
	
	/**
	 * Constructs a new <code>Pulser</code> object with the timer set to time passed in.
	 * @param nTime
	 */
	public Pulser(int nTime)
	{
		setPulseTime(nTime);
	}
	
	/**
	 * Change the delay between the on/off pulse time.
	 * @param nTime <TT>int</TT>
	 */
	public void setPulseTime(int nTime)
	{
		nPulseTime = nTime;
	}
	
	/**
	 * Retrieve the value of the time between the on/off pulse.
	 * @return <TT>int</TT>
	 */
	public int getPulseTime()
	{
		return nPulseTime;
	}
	
	/**
	 * Creates a channel on and off event for a specified amount of time. 
	 * The default PULSE time should be set to 5 (0.5 seconds).
	 * @param dev <code>NetLinxDevice</code>
	 * @param nChannel <TT>int</TT>
	 */
	public void pulse(NetLinxDevice dev, int nChannel)
	{
		if (dev != null)
		{
			// Create a new pulse object and add it to this queue
			devpulse p = new devpulse(dev, nChannel);
			synchronized (p)
			{
				queue.addLast(p);
			}
		}
	}
	
	// Callback method
	// Removes the pulse object off of the queue
	protected synchronized void remove()
	{
		queue.removeFirst();
	}

}
