/*
 * Created on May 22, 2005
 * @author VGonzales
 * 
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.util;

import java.text.DateFormat;
import java.text.Format;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import com.amx.duet.tools.lang.ObjectRef;

/**
 * A utility class, which contains a variety of Time and Date manipulation 
 * routines.
 * 
 * @version 1.0.1
 * @since AMXTools 1.0.0
 *
 */
public class TimeDateUtil
{
//	private static NetLinx m_instance = new NetLinx();
	
	public static final String TIME_FORMAT = "hh:mm:ss";
	public static final String DATE_FORMAT = "MM/dd/yyyy";
	public static final String TIME_DATE_FORMAT = "hh:mm:ss MM/dd/yyyy";
	
	//(***********************************************************)
	//(*               CONSTANT DEFINITIONS GO BELOW             *)
	//(***********************************************************)
	// Length for storage
	public static final int TDL_DATE_LEN    = 20;
	public static final int TDL_TIME_LEN    = 10;
	
	// Days Of The Week 
	public static final int TDL_SUNDAY      = 1;
	public static final int TDL_MONDAY      = 2;
	public static final int TDL_TUESDAY     = 3;
	public static final int TDL_WEDNESDAY   = 4;
	public static final int TDL_THURSDAY    = 5;
	public static final int TDL_FRIDAY      = 6;
	public static final int TDL_SATURDAY    = 7;
	
	// Some Handy Numbers 
	public static final int TDL_SECONDS_PER_MINUTE = 60;
	public static final int TDL_MINUTES_PER_HOUR   = 60;
	public static final int TDL_HOURS_PER_DAY      = 24;
	public static final int TDL_SECONDS_PER_HOUR   = TDL_SECONDS_PER_MINUTE * TDL_MINUTES_PER_HOUR;
	public static final int TDL_MINUTES_PER_DAY    = TDL_MINUTES_PER_HOUR   * TDL_HOURS_PER_DAY;
	public static final int TDL_SECONDS_PER_DAY    = TDL_SECONDS_PER_HOUR   * TDL_HOURS_PER_DAY;
	public static final int TDL_MONTHS_PER_YEAR    = 12;
	public static final int TDL_DAYS_PER_YEAR      = 365;
	
    public static final int  ONE_SECOND = 1000;
    public static final int  ONE_MINUTE = 60*ONE_SECOND;
    public static final int  ONE_HOUR   = 60*ONE_MINUTE;
    public static final long ONE_DAY    = 24*ONE_HOUR;
    public static final long ONE_WEEK   = 7*ONE_DAY;
    
	//(***********************************************************)
	//(*           SUBROUTINE DEFINITIONS GO BELOW               *)
	//(***********************************************************)
	
	/**
	 * Convert the current system time to nano seconds for applications that require it.
	 * 
	 * @return a long value containing the converted time.
	 */
	public final static long nanoTime()
	{
		return ((System.currentTimeMillis() % 1000) * 1000000);
	}

	/**
	 * Retrieves the long name of a day from a Date object
	 * 
	 * @param date a Date object
	 * @return The long name of a Day (ie. Wednesday)
	 */
	public final static String LongDayName (Date date)
	{
		Format formatter = new SimpleDateFormat("EEEE");  // Wednesday
		return formatter.format(date);
	}
	
	/**
	 * Retrieves the short name of a day from a Date object
	 * 
	 * @param date a Date object
	 * @return The short name of a Day (ie. Wed)
	 */
	public final static String ShortDayName (Date date)
	{
		Format formatter = new SimpleDateFormat("E");  // Wed
		return formatter.format(date);
	}
	
	/**
	 * Retrieves the long name of a month from a Date object
	 * 
	 * @param date a Date object
	 * @return The long name of a Month (ie. January)
	 */
	public final static String LongMonthName (Date date)
	{
		Format formatter = new SimpleDateFormat("MMMM");  // January
		return formatter.format(date);
	}
	
	/**
	 * Retrieves the short name of a month from a Date object
	 * 
	 * @param date a Date object
	 * @return The short name of a Month (ie. Jan)
	 */
	public final static String ShortMonthName (Date date)
	{
		Format formatter = new SimpleDateFormat("MMM");  // Jan
		return formatter.format(date);
	}
	
	//(**************************************)
	//(* Call Name: GetFormalDateDay        *)
	//(* Function:  Return Formal Date Day  *)
	//(* Param:     Day                     *)
	//(* Return:    Formal Day String       *)
	//(**************************************)
	public final static String GetFormalDateDay(long lDay)
	{
		// Valid? 
		if (lDay <= 0 || lDay > 31)
			return "";
	
		switch ((int)lDay)
		{
			case  1: return lDay + "st";
			case 21: return lDay + "st";
			case 31: return lDay + "st";
			case  2: return lDay + "nd";
			case 22: return lDay + "nd";
			case  3: return lDay + "rd";
			case 23: return lDay + "rd";
			default: return lDay + "th";
		}
	}
	
	//(**************************************)
	//(* Call Name: GetFormalDayOffset      *)
	//(* Function:  Return Formal Date Day  *)
	//(* Param:     Day                     *)
	//(* Return:    Formal Day String       *)
	//(**************************************)
	public final static String GetFormalDayOffset(long lOffset)
	{
		// Valid? 
		if (lOffset <= 0 || lOffset > 5)
			return "";
	
		switch ((int)lOffset)
		{
			case 1: return "First";
			case 2: return "Second";
			case 3: return "Third";
			case 4: return "Fourth";
			case 5: return "Last";
		}
		return "";
	}
	
	//(**************************************)
	//(* Call Name: GetFormalDate           *)
	//(* Function:  Return Formal Date      *)
	//(* Param:     Date String             *)
	//(* Return:    Formal Date String      *)
	//(**************************************)
	public final static String GetFormalDate (String sDate, boolean bUKDate) throws ParseException
	{
		long lMonth = 0;
		long lDay = 0;
		long lYear = 0;
		
		// Anything To Do? 
		if (sDate.length() == 0)
			return "";
	
		Date date = DateStringToDate(sDate);
	
		// Parts
		lMonth = DateToMonth(sDate);
		lDay   = DateToDay(sDate);
		lYear  = DateToYear(sDate);
		if (lMonth < 0 || lDay <= 0 || lYear <= 0)
			return "";
		if (lMonth >= 12 || lDay > DaysPerMonth(lMonth,lYear))
			return "";
	
		// Do It 
		if (bUKDate)
			return GetFormalDateDay(lDay) + " " + LongMonthName(date) + ", " + lYear;
		return LongMonthName(date) + " " + GetFormalDateDay(lDay) + ", " + lYear;
	}
	
	//(**************************************)
	//(* Call Name: Dayspermonth            *)
	//(* Function:  Return Just 'dat        *)
	//(* Param:     Month, Year             *)
	//(* Return:    Days Per Month          *)
	//(* Note:      Super Hack! Allows      *)
	//(*            Month To Be Outisde     *)
	//(*            Normal Range And New    *)
	//(*            Year Is Calc'ed         *)
	//(**************************************)
	public final static int DaysPerMonth(long lMonth, long lYear)
	{
	    // Create a calendar object of the desired month
	    Calendar cal = new GregorianCalendar((int)lYear, (int)lMonth, 1);
	    
	    // Get the number of days in that month
	    return cal.getActualMaximum(Calendar.DAY_OF_MONTH); // 28
	}
	
	//(**************************************)
	//(* Call Name: TimeSerial              *)
	//(* Function:  Serialize Time          *)
	//(* Param:     Hour, On, Sec           *)
	//(* Return:    Time                    *)
	//(**************************************)
	public final static String TimeSerial (long lHour, long lMin, long lSec)
	{
		// Sanity 
		if (lHour > 23 || lMin > 59 || lSec > 59)
			return "";
	
		return FormatTime((int)lHour, (int)lMin, (int)lSec);
	}
	
	//(**************************************)
	//(* Call Name: DateSerial              *)
	//(* Function:  Serialize Date          *)
	//(* Param:     Month, Day, Year        *)
	//(*            Year Length             *)
	//(* Return:    Date                    *)
	//(**************************************)
	public final static String DateSerial (long lMonth, long lDay, long lYear, int nYrLen, boolean bUKDate)
	{
		// Sanity 
		if (lMonth < 0 || lMonth >= 12 || lDay == 0)
			return "";
		if (lDay > DaysPerMonth(lMonth, lYear))
			return "";
	
		// Do Year Length 
		//nYearLen = nYrLen;
		String sYear = "";
		if (nYrLen == 2)
			sYear = "yy";
		else
			sYear = "yyyy";
			
		String sDate = "";
		if (bUKDate)
			sDate = FormatDate((int)lMonth, (int)lDay, (int)lYear, "dd/MM/" + sYear);
		else
			sDate = FormatDate((int)lMonth, (int)lDay, (int)lYear, "MM/dd/" + sYear);
		
		return sDate;
	}
	
	//(**************************************)
	//(* Call Name: LocalizedDate           *)
	//(* Function:  Return Date In Us Or Uk *)
	//(*            Format                  *)
	//(* Param:     Date                    *)
	//(* Return:    Date                    *)
	//(**************************************)
	public final static String LocalizedDate (String sDate, boolean bUKDate)
	{
		long lMonth = 0;
		long lDay = 0;
		long lYear = 0;
	
		// Anything To Do? 
		if (sDate.length() == 0)
			return "";
	
		// Parts 
		lMonth = DateToMonth(sDate);
		lDay   = DateToDay(sDate);
		lYear  = DateToYear(sDate);
		if (lMonth < 0 || lDay <= 0 || lYear <= 0)
			return "";
		if (lMonth >= 12 || lDay > DaysPerMonth(lMonth,lYear))
			return "";
	
		// Netlinx Is Native MM/DD/YYYY 
		// Go ahead and let this be serialzed. LocalizedDate() can then validate a date 
		//if (!bUKDate)
		//	return sDate;
	
		// Return DD/MM/YYYY 
		return DateSerial(lMonth, lDay, lYear, 4, bUKDate);
	}
	
	//(**************************************)
	//(* Call Name: TimeAdd                 *)
	//(* Function:  Adjust Times            *)
	//(* Param:     Time, Sec, Min, Hour,   *)
	//(*            Day Offset              *)
	//(* Return:    New Time                *)
	//(**************************************)
	public final static String TimeAdd (String sTime, long lSec, long lMin, long lHour)
	{
		long lNowMin = 0;
		long lNowHour = 0;
		long lNowSec = 0;
	
		// Break Up Components 
		lNowSec  = TimeToSecond(sTime) + lSec;
		lNowMin  = TimeToMinute(sTime) + lMin;
		lNowHour = TimeToHour(sTime)   + lHour;
		
		// Now, Put It Back Together 
		return TimeSerial(lNowHour, lNowMin, lNowSec);
	}
	
	//(**************************************)
	//(* Call Name: Time Diff               *)
	//(* Function:  Difference Between Times*)
	//(* Param:     Time1, Time2, Sec, Min, *)
	//(*            Hour, Ctime, T1 < T2    *)
	//(* Return:    0 = Good                *)
	//(*           -1 = Time1 Invalid       *)
	//(*           -2 = Time2 Invalid       *)
	//(* Notes:    Nsec, Nmin And nHour     *)
	//(*           Are Totals,  sTimeDiff   *)
	//(*           Is A Difference Time Str *)
	//(**************************************)
	public final static String TimeDiff (String sTime1, String sTime2, ObjectRef rlSec, ObjectRef rlMin, ObjectRef rlHour, ObjectRef rbOneLtTwo) throws ParseException
	{
		Date time1 = TimeStringToDate(sTime1);
		Date time2 = TimeStringToDate(sTime2);
		
	    // Determine which is earlier
	    rbOneLtTwo.valueOf(time1.before(time2));
	    
	    // Get difference in milliseconds
	    long diffMillis = time2.getTime()-time1.getTime();

	    // Get difference in seconds
	    long diffSecs = diffMillis/(ONE_SECOND);
	    rlSec.valueOf(diffSecs);
	    
	    // Get difference in minutes
	    long diffMins = diffMillis/(ONE_MINUTE);
	    rlMin.valueOf(diffSecs);
	    
	    // Get difference in hours
	    long diffHours = diffMillis/(ONE_HOUR);
	    rlHour.valueOf(diffSecs);
	    
		return TimeSerial(diffHours, diffMins, diffSecs);
	}

	public final static int DateDiff (String sDate1, String sDate2, ObjectRef rDay, ObjectRef rMonth, ObjectRef rYear)
	{
		long slNowDay1 = 0;
		long slNowMonth1 = 0;
		long slNowYear1 = 0;
		long slNowDay2 = 0;
		long slNowMonth2 = 0;
		long slNowYear2 = 0;
		long slDay = 0;
		long slMonth = 0;
		long slYear = 0;
		boolean bSwapDates = false;
	
		// Break Up Components
		slNowDay1   = DateToDay(sDate1);
		slNowMonth1 = DateToMonth(sDate1);
		slNowYear1  = DateToYear(sDate1);
		
		if (slNowYear1 <= 0 || slNowMonth1 <= 0 || slNowDay1 <= 0)
			return -1;
	
		// Break Up Components
		slNowDay2   = DateToDay(sDate2);
		slNowMonth2 = DateToMonth(sDate2);
		slNowYear2  = DateToYear(sDate2);
		
		if (slNowYear2 <= 0 || slNowMonth2 <= 0 || slNowDay2 <= 0)
			return -2;
	    
		// Swap dates?
		if (slNowYear1 < slNowYear2)
			bSwapDates = true;
		else if (slNowYear1 > slNowYear2)
			bSwapDates = false;
		else if (slNowMonth1 < slNowMonth2)
			bSwapDates = true;
		else if (slNowMonth1 > slNowMonth2)
			bSwapDates = false;
		else if (slNowDay1 < slNowDay2)
			bSwapDates = true;
		else
			bSwapDates = false;
		
		if (bSwapDates)
		{
			slNowDay1   = DateToDay(sDate2);
			slNowMonth1 = DateToMonth(sDate2);
			slNowYear1  = DateToYear(sDate2);
	    
			slNowDay2   = DateToDay(sDate1);
			slNowMonth2 = DateToMonth(sDate1);
			slNowYear2  = DateToYear(sDate1);
		}
	
		// Now Calc Diff
		// Year Will Not Be -
		slYear  = slNowYear1 - slNowYear2;
		slMonth = slNowMonth1 - slNowMonth2;
		
		if (slMonth < 0)
		{
			// If This Happens, It Is Because slYear >= 1
			slYear--;
			slMonth = slMonth + 12;
		}
		
		slDay   = slNowDay1 - slNowDay2;
		slMonth = slMonth + (slYear * 12);
		
		if (slDay < 0)
			slMonth--;
		if (slMonth < 0)
			slYear--;
	
		// Wow, Days Is A Bit Harder Than Those... Why Is There A Variable Number Of Days Per Month?
		// Negative slDay Get Fixed Here Since We Include Days Per Month Of This Month...
		while (slNowYear2 < slNowYear1 || slNowMonth2 < slNowMonth1)
		{
			//slDay = slDay + TimerDaysPerMonth(slNowMonth2, slNowYear2);
			slDay = slDay + DaysPerMonth(slNowMonth2, slNowYear2);
			slNowMonth2++;
			if (slNowMonth2 > 12)
			{
				slNowMonth2 = 1;
				slNowYear2++;
			}
		}
		
		// Copy
		rDay.valueOf(slDay);
		rMonth.valueOf(slMonth);
		rYear.valueOf(slYear);
		
		return 0;
	}

//	public final static String DateDiff (String sDate1, String sDate2, ObjectRef rlDay, ObjectRef rlMonth, ObjectRef rlYear, ObjectRef rbOneLtTwo) throws ParseException
//	{
//		Date date1 = DateStringToDate(sDate1);
//		Date date2 = DateStringToDate(sDate2);
//		boolean bOneLtTwo = false;
//
//	    // Determine which is earlier
//	    bOneLtTwo = date1.before(date2);
//		
//		long diffYears  = TimeDateDiff(sDate1, sDate2, Calendar.DAY_OF_MONTH, rbOneLtTwo);
//		long diffMonths = TimeDateDiff(sDate1, sDate2, Calendar.MONTH, rbOneLtTwo);
//		long diffDays   = TimeDateDiff(sDate1, sDate2, Calendar.YEAR, rbOneLtTwo);
//		
//		rlDay.valueOf(diffDays);
//		rlMonth.valueOf(diffMonths);
//		rlYear.valueOf(diffYears);
//
//	    rbOneLtTwo.valueOf(bOneLtTwo);
//		
//		return DateSerial(diffDays, diffMonths, diffYears, 4, false);
//	}
	
	//(**************************************)
	//(* Call Name: DateAdd                 *)
	//(* Function:  Adjust Dates            *)
	//(* Param:     Date, Day, Month, Year  *)
	//(* Return:    New Date                *)
	//(**************************************)
	public final static String DateAdd (String sDate, long lDay, long lMonth, long lYear)
	{
		long lNowDay = 0;
		long lNowMonth = 0;
		long lNowYear = 0;
	
		// Break Up Components 
		lNowDay   = DateToDay(sDate)   + lDay;
		lNowMonth = DateToMonth(sDate) + lMonth;
		lNowYear  = DateToYear(sDate)  + lYear;
		
		// Now, Put It Back Together 
		return DateSerial(lNowMonth, lNowDay, lNowYear, 4, false);
	}
	
	//(**************************************)
	//(* Call Name: DateCompare             *)
	//(* Function:  Compare Dates           *)
	//(* Param:     Date1, Date2            *)
	//(* Return:    0 = Equal               *)
	//(*           -1 = Date1 < Date2       *)
	//(*            1 = Date2 < Date1       *)
	//(* Notes:    Date Are Not Check For   *)
	//(*           Validity                 *)
	//(**************************************)
	public final static int DateCompare (String sDate1, String sDate2) throws ParseException
	{
		Date date1 = DateStringToDate(sDate1);
		Date date2 = DateStringToDate(sDate2);
		
	    // Determine which is earlier
	    if (date1.before(date2)) return -1;
	    if (date1.after(date2))  return  1;
	    return 0;
	}
	
	//(**************************************)
	//(* Call Name: AmPmTime                *)
	//(* Function:  Return Am/Pm Time Str   *)
	//(* Param:     Time                    *)
	//(* Return:    Am/Pm Time              *)
	//(**************************************)
	public final static String AmPmTime (String sTime) throws ParseException
	{
		Date time = TimeStringToDate(sTime);
		return DateToString(time, "hh:mm:ss a");
	}
	
	//(**************************************)
	//(* Call Name: ShortAmPmTime           *)
	//(* Function:  Return Am/Pm Time Str   *)
	//(* Param:     Time                    *)
	//(* Return:    Am/Pm Time              *)
	//(**************************************)
	public final static String ShortAmPmTime (String sTime) throws ParseException
	{
		Date time = TimeStringToDate(sTime);
	    return DateToString(time, "hh:mm a");
	}
	
	
	//(**************************************)
	//(* Call Name: LocalizedTime           *)
	//(* Function:  Return Time In 12 Or 24 *)
	//(*            Hour Format             *)
	//(* Param:     Time                    *)
	//(* Return:    Time                    *)
	//(**************************************)
	public final static String LocalizedTime (String sTime, boolean bMilitary) throws ParseException
	{
		// Anything To Do? 
		if (sTime.length() == 0)
			return "";
	
		// Netlinx Is Native HH/MM/SS In 24 Hour Format 
		// However, serialize it so as to validate it.
		if (bMilitary)
		{
			Date time = DateStringToDate(sTime);
			return DateToString(time, "HH:mm:ss a");
		}
	
		// Return HH/MM/SS Am/Pm 
		return AmPmTime(sTime);
	}
	
	//(**************************************)
	//(* Call Name: ShortLocalizedTime      *)
	//(* Function:  Return Time In 12 Or 24 *)
	//(*            Hour Format             *)
	//(* Param:     Time                    *)
	//(* Return:    Time                    *)
	//(**************************************)
	public final static String ShortLocalizedTime (String sTime, boolean bMilitary) throws ParseException
	{
		// Anything To Do? 
		if (sTime.length() == 0)
			return "";
	
		// Netlinx Is Native HH/MM In 24 Hour Format 
		// However, serialize it so as to validate it.
		if (bMilitary)
		{
			Date time = DateStringToDate(sTime);
			return DateToString(time, "HH:mm a");
		}
	
		// Return HH/MM Am/Pm 
		return ShortAmPmTime (sTime);
	}
	
	//(*****************************************)
	//(* Call Name: CalcFloatingDate           *)
	//(* Function:  Calculate Date For A       *)
	//(*            Floating Date Spec         *)
	//(* Param:     Day Offset (1,2,3,4,5=Last *)
	//(*            Day Of Week (1=Sun)        *)
	//(*            Month                      *)
	//(*            Year                       *)
	//(* Return:    Date (4 Digit Year)        *)
	//(*****************************************)
	public final static String CalcFloatingDate(int nDayOfMonthOffset, int nDayOfMonthDay, int nMonth, int nYear) throws ParseException
	{
		String sTempDate = "";
		int nDOW = 0;
		int nPos = 0;
	
		// Get 1st Day Of This Month 
		sTempDate = DateSerial(nMonth, 1, nYear, 4, false);
	
		// What Day Of Week That Is	
		Date date = DateStringToDate(sTempDate);
	    Calendar cal = new GregorianCalendar();
	    cal.setTime(date);
	    nDOW = cal.get(Calendar.DAY_OF_WEEK);
	
		// Figure Out You Many Days From Start Of Month To Our Desired Day 
		nPos = nDayOfMonthDay;
	
		// If Our Desird Is A Daw Week Before Dow For 1St Of The Month, Give Us And Exra 7 Days To Next Calc Is Right 
		if (nPos < nDOW) nPos = nPos + 7;
	
		// Number Of Days To Get The First Occurance Of Our Desired Day Of Week 
		nPos = nPos - nDOW;
	
		// Add Number Of Days To Our Date - First Of Our Target Month 
		if (nPos > 0)
			sTempDate = DateAdd(sTempDate, nPos, 0, 0);
	
		// Now If It Is A Multiple Offset (Like Second, Third, Fourth, Or Last, Add That Many * 7 
		if (nDayOfMonthOffset > 1)
		{
			// Start At 1 Less (We Already Have The First 
			// Is Last A Fifth Or Fourth? 
			nDOW = nDayOfMonthOffset - 1;
			if (nDOW == 4)
			{
				// nPos Hodls Date Of First Occurance Of Deired Day So Add 4 Weeks To See Where We Land 
				// If This Date Is Higher Than Number Of Days In Month Then Last Is The Fourth Occurance 
				nPos = nPos + 28;
				if (nPos > DaysPerMonth (nMonth, nYear))
					nDOW--;
			}
			// Now, Calc Number Of Days To Add And Do It 
			nPos = (nDOW * 7);
			sTempDate = DateAdd(sTempDate, nPos, 0, 0);
		}
	
		// Done 
		return sTempDate;
	}

	/**
	 * Converts a string to a date object
	 * @param str
	 * @param pattern
	 * @return
	 * @throws ParseException
	 */
	public final static Date StringToDate(String str, String pattern) throws ParseException
	{
		Date dt = null;
		
		SimpleDateFormat dfTime = new SimpleDateFormat(pattern);
		if (dfTime != null)
			dt = dfTime.parse(str);
		
		return dt;
	}
	
	/**
	 * Converts a date object to a string
	 * @param date
	 * @param pattern
	 * @return
	 */
	public final static String DateToString(Date date, String pattern)
	{
	    // Some examples
	    Format formatter = new SimpleDateFormat(pattern);
	    return formatter.format(date);
	}
	
	/**
	 * Converts a preformatted date string to a date object
	 * @param sDate
	 * @return
	 * @throws ParseException
	 */
	public final static Date DateStringToDate(String sDate) throws ParseException
	{
		Date date = null;
		try
		{
			date = StringToDate(sDate, DATE_FORMAT);
		}
		catch (ParseException e)
		{
			try
			{
				DateFormat dfTime = DateFormat.getDateInstance(DateFormat.SHORT);
				if (dfTime != null)
					date = dfTime.parse(sDate);
			}
			catch (ParseException pe)
			{
				throw pe;
			}
		}

		return date;
	}
	
	/**
	 * Converts a preformatted time string to a date object
	 * @param sTime
	 * @return
	 * @throws ParseException
	 */
	public final static Date TimeStringToDate(String sTime) throws ParseException
	{
		return StringToDate(sTime, TIME_FORMAT);
	}
	
	/**
	 * Converts a date object to a preformatted date string
	 * @param date
	 * @return
	 */
	public final static String DateToStringDate(Date date)
	{
	    return DateToString(date, DATE_FORMAT);
	}
	
	/**
	 * Converts a date object to a preformatted time string
	 * @param date
	 * @return
	 */
	public final static String DateToStringTime(Date date)
	{
	    return DateToString(date, TIME_FORMAT);
	}
	
	/**
	 * Formats the integer values of a date to a preformatted date string
	 * @param nMonth
	 * @param nDay
	 * @param nYear
	 * @return
	 */
	public final static String FormatDate(int nMonth, int nDay, int nYear)
	{
	    Calendar cal = new GregorianCalendar(nYear, nMonth, nDay);
		return DateToStringDate(cal.getTime());
	}

	public final static String FormatDate(int nMonth, int nDay, int nYear, String sPattern)
	{
	    Calendar cal = new GregorianCalendar(nYear, nMonth, nDay);
		return DateToString(cal.getTime(), sPattern);
	}
	
	/**
	 * Formats the integer values of a time to a preformatted time string
	 * @param nHour
	 * @param nMin
	 * @param nSec
	 * @return
	 */
	public final static String FormatTime(int nHour, int nMin, int nSec)
	{
	    Calendar cal = new GregorianCalendar();
	    cal.set(Calendar.HOUR_OF_DAY, nHour);
	    cal.set(Calendar.MINUTE, nMin);
	    cal.set(Calendar.SECOND, nSec);
		return DateToStringTime(cal.getTime());
	}
	
	public final static long TimeToSecond(String sTime)
	{
		long lSeconds = 0;
		
		try
		{
			Date dt = TimeStringToDate(sTime);
			if (dt != null)
			{
			    Calendar calendar = new GregorianCalendar();
				calendar.setTime(dt);
				lSeconds = calendar.get(Calendar.SECOND);
			}
		}
		catch (ParseException e)
		{
			error("TimeToSecond(): ParseException thrown! " + e.getMessage());
		}
		
		return lSeconds;
	}

	public final static long TimeToMinute(String sTime)
	{
		long lMinutes = 0;
		
		try
		{
			Date dt = TimeStringToDate(sTime);
			if (dt != null)
			{
			    Calendar calendar = new GregorianCalendar();
				calendar.setTime(dt);
				lMinutes = calendar.get(Calendar.MINUTE);
			}
		}
		catch (ParseException e)
		{
			error("TimeToMinute(): ParseException thrown! " + e.getMessage());
		}
		
		return lMinutes;
	}

	public final static long TimeToHour(String sTime)
	{
		long lHour = 0;
		
		try
		{
			Date dt = TimeStringToDate(sTime);
			if (dt != null)
			{
			    Calendar calendar = new GregorianCalendar();
				calendar.setTime(dt);
				lHour = calendar.get(Calendar.HOUR);
			}
		}
		catch (ParseException e)
		{
			error("TimeToHour(): ParseException thrown! " + e.getMessage());
		}
		
		return lHour;
	}
	
	public final static long DateToDay(String sDate)
	{
		long lDay = 0;
		
		try
		{
			Date dt = DateStringToDate(sDate);
			if (dt != null)
			{
			    Calendar calendar = new GregorianCalendar();
				calendar.setTime(dt);
				lDay = calendar.get(Calendar.DAY_OF_MONTH);
			}
		}
		catch (ParseException e)
		{
			error("DateToDay(): ParseException thrown! " + e.getMessage());
		}
		
		return lDay;
	}

	public final static long DateToMonth(String sDate)
	{
		long lMonth = 0;
		
		try
		{
			Date dt = DateStringToDate(sDate);
			if (dt != null)
			{
			    Calendar calendar = new GregorianCalendar();
				calendar.setTime(dt);
				lMonth = calendar.get(Calendar.MONTH);
			}
		}
		catch (ParseException e)
		{
			error("DateToMonth(): ParseException thrown! " + e.getMessage());
		}
		
		return lMonth;
	}

	public final static long DateToYear(String sDate)
	{
		long lYear = 0;
		
		try
		{
			Date dt = DateStringToDate(sDate);
			if (dt != null)
			{
			    Calendar calendar = new GregorianCalendar();
				calendar.setTime(dt);
				lYear = calendar.get(Calendar.YEAR);
			}
		}
		catch (ParseException e)
		{
			error("DateToYear(): ParseException thrown! " + e.getMessage());
		}
		
		return lYear;
	}
	
	public final static String CurrentDate()
	{
	    // Create a calendar object with today's date
	    Calendar cal = Calendar.getInstance();
		return FormatDate(cal.get(Calendar.MONTH), cal.get(Calendar.DAY_OF_MONTH), cal.get(Calendar.YEAR));
	}

	public final static String CurrentTime()
	{
	    // Create a calendar object with today's date
	    Calendar cal = Calendar.getInstance();
		return FormatTime(cal.get(Calendar.HOUR), cal.get(Calendar.MINUTE), cal.get(Calendar.SECOND));
	}
	
	/**
	 * Gets the select difference between two time values
	 * 
	 * @param sTimeDate1 must be in either (MM/DD/YYYY) date format or (hh:mm:ss) time format
	 * @param sTimeDate2 must be in either (MM/DD/YYYY) date format or (hh:mm:ss) time format
	 * @param nTimeField <TT>Calendar</TT> field modifier (ie Calendar.YEAR, Calendar.HOUR)
	 * @param rbOneLtTwo must be a boolean type return value
	 * 
	 * @return a <TT>long</TT> value containing the time difference based on the field 
	 * modifier passed in
	 * @throws ParseException
	 */
	public final static long TimeDateDiff (String sTimeDate1, String sTimeDate2, int nTimeField, ObjectRef rbOneLtTwo) throws ParseException
	{
		long diffValue = 0;

		Date timedate1 = null;
		Date timedate2 = null;
		try
		{
			timedate1 = DateStringToDate(sTimeDate1);
			timedate2 = DateStringToDate(sTimeDate2);
		}
		catch (ParseException e)
		{
			timedate1 = TimeStringToDate(sTimeDate1);
			timedate2 = TimeStringToDate(sTimeDate2);
		}

		switch (nTimeField)
		{
			case Calendar.DAY_OF_MONTH:
			{
			    // Determine which is earlier
			    rbOneLtTwo.valueOf(timedate1.before(timedate2));

			    // Get difference in milliseconds
			    long diffMillis = timedate2.getTime()-timedate1.getTime();

			    // Get difference in days
			    diffValue = Math.abs(diffMillis/(24*60*60*1000));
				break;
			}
			case Calendar.MONTH:
			{
				long lNowMonth1 = DateToMonth(sTimeDate1);
				long lNowYear1  = DateToYear(sTimeDate1);
		
				long lNowMonth2 = DateToMonth(sTimeDate2);
				long lNowYear2  = DateToYear(sTimeDate2);

			    // Determine which is earlier
			    rbOneLtTwo.valueOf(timedate1.before(timedate2));

				long diffYears = lNowYear2 - lNowYear1;
				diffValue = lNowMonth2 - lNowMonth1;
				diffValue += Math.abs(diffYears*12);
				break;
			}
			case Calendar.YEAR:
			{
				long lNowYear1  = DateToYear(sTimeDate1);
				long lNowYear2  = DateToYear(sTimeDate2);
				
			    // Determine which is earlier
			    rbOneLtTwo.valueOf(timedate1.before(timedate2));

				diffValue = Math.abs(lNowYear2 - lNowYear1);
				break;
			}
			case Calendar.SECOND:
			{
			    // Determine which is earlier
			    rbOneLtTwo.valueOf(timedate1.before(timedate2));

			    // Get difference in milliseconds
			    long diffMillis = timedate2.getTime()-timedate1.getTime();

			    // Get difference in seconds
			    diffValue = Math.abs(diffMillis/(ONE_SECOND));
				break;
			}
			case Calendar.MINUTE:
			{
			    // Determine which is earlier
			    rbOneLtTwo.valueOf(timedate1.before(timedate2));

			    // Get difference in milliseconds
			    long diffMillis = timedate2.getTime()-timedate1.getTime();

			    // Get difference in minutes
			    diffValue = Math.abs(diffMillis/(ONE_MINUTE));
				break;
			}
			case Calendar.HOUR:
			{
			    // Determine which is earlier
			    rbOneLtTwo.valueOf(timedate1.before(timedate2));

			    // Get difference in milliseconds
			    long diffMillis = timedate2.getTime()-timedate1.getTime();

			    // Get difference in hours
			    diffValue = Math.abs(diffMillis/(ONE_HOUR));
				break;
			}
		}
		
		return diffValue;
	}

	/**
	 * Adds the specified number of milliseconds to the current time and returns
	 * the added value
	 * 
	 * @param timeout number of milliseconds
	 * @return adjusted time in milliseconds
	 */
	public final static long getNextTimeout(long timeout)
	{
		long nextTimeout = 0;
		Calendar now = new GregorianCalendar();
		Date nowTime = now.getTime();
		nextTimeout = nowTime.getTime() + timeout;
		
		return nextTimeout;
	}
	
	/**
	 * Checks to see if the current time is greater than the specified time in 
	 * milliseconds
	 * 
	 * @param timeout time in milliseconds
	 * @return true is the time is expired; false otherwise
	 */
	public final static boolean isTimeExpired(long timeout)
	{
		boolean bExpired = false;
		
		Calendar now = new GregorianCalendar();
		Date nowTime = now.getTime();
		Date dt = new Date(timeout);
		if (nowTime.after(dt))
			bExpired = true;
		
		return bExpired;
	}

	private final static void error(String str)
	{
		System.out.println("TimeDateUtil: ERROR - " + str);
	}
}
