/*
 * @author VGonzales
 * Created on May 23, 2005
 * @since 1.0.0 
 * @version 1.0.0, 05/23/05
 *  
 ****************************************************************
 * Copyright Notice :
 * Copyright, AMX, Inc., 2006-2008
 * Private, proprietary information, the sole property of AMX.
 * The contents, ideas, and concepts expressed herein are not to
 * be disclosed except within the confines of a confidential
 * relationship and only then on a need to know basis.    
 ****************************************************************
 */
package com.amx.duet.tools.xml;

/**
 * The <code>Attribute</code> class was created to hold attributes for XML {@link Tag} objects, 
 * but it also has the capabilities to hold HTML attributes. This class can be further 
 * overwritten to handle the attribute of the user's choice.
 * 
 * @version 1.0.0
 * @since AMXTools 1.0.0
 * 
 */
public class Attribute
{
	private String name = "";
	private String namespace = "";
	private String prefix = "";
	private String type = "";
	private String value = "";
	
	/**
	 * Ensure that we will always have an attribute name
	 * @param sName <TT>String</TT>
	 */
	public Attribute(String sName)
	{
		setName(sName);
	}
	
	/**
	 * Set the name for this attribute
	 * @param sName <TT>String</TT>
	 */
	public void setName(String sName) { name = sName; }
	/**
	 * Set the name space for this attribute
	 * @param sNameSpace <TT>String</TT>
	 */
	public void setNameSpace(String sNameSpace) { namespace = sNameSpace; }
	/**
	 * Set the prefix value for this attribute
	 * @param sPrefix <TT>String</TT>
	 */
	public void setPrefix(String sPrefix) { prefix = sPrefix; }
	/**
	 * Set the type for the value of this attribute
	 * @param sType <TT>String</TT>
	 */
	public void setType(String sType) { type = sType; }
	/**
	 * Set a value for this attribute
	 * @param sValue <TT>String</TT>
	 */
	public void setValue(String sValue) { value = sValue; }
	
	/**
	 * Retrieve the name of this attribute
	 * @return <TT>String</TT>
	 */
	public String getName() { return name; }
	/**
	 * Retrieve the name space for this attribute
	 * @return <TT>String</TT>
	 */
	public String getNameSpace() { return namespace; }
	/**
	 * Retrieve the prefix for this attribute
	 * @return <TT>String</TT>
	 */
	public String getPrefix() { return prefix; }
	/**
	 * Retrieve the type for the value of this attribute
	 * @return <TT>String</TT>
	 */
	public String getType() { return type; }
	/**
	 * Retrieve the value stored in this attribute
	 * @return <TT>String</TT>
	 */
	public String getValue() { return value; }
	/**
	 * Outputs all of the attribute data to the print stream
	 */
	public void dump()
	{
		System.out.println("Attribute: ========= Attributes ===========");
		System.out.println("Attribute: Name:          " + name);
		System.out.println("Attribute: Type:          " + type);
		System.out.println("Attribute: Value:         " + value);
		System.out.println("Attribute: Name space:    " + namespace);
		System.out.println("Attribute: Prefix:        " + prefix);
		System.out.println("Attribute: ================================");
	}
}
